@@ -31,8 +31,10 @@
 #include <extensionsystem/pluginmanager.h>
 #include <extensionsystem/pluginspec.h>
 #include <qtsingleapplication.h>
+
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
+#include <utils/temporarydirectory.h>
 
 #include <QDebug>
 #include <QDir>
@@ -56,6 +58,10 @@
 #include <QStandardPaths>
 #include <QTemporaryDir>
 
+#include <string>
+#include <vector>
+#include <iterator>
+
 #ifdef ENABLE_QT_BREAKPAD
 #include <qtsystemexceptionhandler.h>
 #endif
@@ -64,7 +70,6 @@ using namespace ExtensionSystem;
 
 enum { OptionIndent = 4, DescriptionIndent = 34 };
 
-const char appNameC[] = "Qt Creator";
 const char corePluginNameC[] = "Core";
 const char fixedOptionsC[] =
 " [OPTION]... [FILE]...\n"
@@ -73,6 +78,7 @@ const char fixedOptionsC[] =
 "    -version                      Display program version\n"
 "    -client                       Attempt to connect to already running first instance\n"
 "    -settingspath <path>          Override the default path where user settings are stored\n"
+"    -installsettingspath <path>   Override the default path from where user-independent settings are read\n"
 "    -pid <pid>                    Attempt to connect to instance given by pid\n"
 "    -block                        Block until editor is closed\n"
 "    -pluginpath <path>            Add a custom search path for plugins\n";
@@ -84,6 +90,7 @@ const char HELP_OPTION4[] = "--help";
 const char VERSION_OPTION[] = "-version";
 const char CLIENT_OPTION[] = "-client";
 const char SETTINGS_OPTION[] = "-settingspath";
+const char INSTALL_SETTINGS_OPTION[] = "-installsettingspath";
 const char TEST_OPTION[] = "-test";
 const char PID_OPTION[] = "-pid";
 const char BLOCK_OPTION[] = "-block";
@@ -108,7 +115,7 @@ static inline QString toHtml(const QString &t)
 static void displayHelpText(const QString &t)
 {
     if (Utils::HostOsInfo::isWindowsHost())
-        QMessageBox::information(0, QLatin1String(appNameC), toHtml(t));
+        QMessageBox::information(0, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), toHtml(t));
     else
         qWarning("%s", qPrintable(t));
 }
@@ -116,7 +123,7 @@ static void displayHelpText(const QString &t)
 static void displayError(const QString &t)
 {
     if (Utils::HostOsInfo::isWindowsHost())
-        QMessageBox::critical(0, QLatin1String(appNameC), t);
+        QMessageBox::critical(0, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), t);
     else
         qCritical("%s", qPrintable(t));
 }
@@ -125,7 +132,7 @@ static void printVersion(const PluginSpec *coreplugin)
 {
     QString version;
     QTextStream str(&version);
-    str << '\n' << appNameC << ' ' << coreplugin->version()<< " based on Qt " << qVersion() << "\n\n";
+    str << '\n' << Core::Constants::IDE_DISPLAY_NAME << ' ' << coreplugin->version()<< " based on Qt " << qVersion() << "\n\n";
     PluginManager::formatPluginVersions(str);
     str << '\n' << coreplugin->copyright() << '\n';
     displayHelpText(version);
@@ -141,6 +148,24 @@ static void printHelp(const QString &a0)
     displayHelpText(help);
 }
 
+QString applicationDirPath(char *arg = 0)
+{
+    static QString dir;
+
+    if (arg)
+        dir = QFileInfo(QString::fromLocal8Bit(arg)).dir().absolutePath();
+
+    if (QCoreApplication::instance())
+        return QApplication::applicationDirPath();
+
+    return dir;
+}
+
+static QString resourcePath()
+{
+    return QDir::cleanPath(applicationDirPath() + '/' + RELATIVE_DATA_PATH);
+}
+
 static inline QString msgCoreLoadFailure(const QString &why)
 {
     return QCoreApplication::translate("Application", "Failed to load core: %1").arg(why);
@@ -149,32 +174,12 @@ static inline QString msgCoreLoadFailure(const QString &why)
 static inline int askMsgSendFailed()
 {
     return QMessageBox::question(0, QApplication::translate("Application","Could not send message"),
-                                 QCoreApplication::translate("Application", "Unable to send command line arguments to the already running instance. "
-                                                             "It appears to be not responding. Do you want to start a new instance of Creator?"),
-                                 QMessageBox::Yes | QMessageBox::No | QMessageBox::Retry,
-                                 QMessageBox::Retry);
-}
-
-static const char *setHighDpiEnvironmentVariable()
-{
-    const char* envVarName = 0;
-    static const char ENV_VAR_QT_DEVICE_PIXEL_RATIO[] = "QT_DEVICE_PIXEL_RATIO";
-#if (QT_VERSION < QT_VERSION_CHECK(5, 6, 0))
-    if (Utils::HostOsInfo().isWindowsHost()
-            && !qEnvironmentVariableIsSet(ENV_VAR_QT_DEVICE_PIXEL_RATIO)) {
-        envVarName = ENV_VAR_QT_DEVICE_PIXEL_RATIO;
-        qputenv(envVarName, "auto");
-    }
-#else
-    if (Utils::HostOsInfo().isWindowsHost()
-            && !qEnvironmentVariableIsSet(ENV_VAR_QT_DEVICE_PIXEL_RATIO) // legacy in 5.6, but still functional
-            && !qEnvironmentVariableIsSet("QT_AUTO_SCREEN_SCALE_FACTOR")
-            && !qEnvironmentVariableIsSet("QT_SCALE_FACTOR")
-            && !qEnvironmentVariableIsSet("QT_SCREEN_SCALE_FACTORS")) {
-        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
-    }
-#endif // < Qt 5.6
-    return envVarName;
+                QCoreApplication::translate("Application", "Unable to send command line arguments "
+                                            "to the already running instance. It does not appear to "
+                                            "be responding. Do you want to start a new instance of "
+                                            "%1?").arg(Core::Constants::IDE_DISPLAY_NAME),
+                QMessageBox::Yes | QMessageBox::No | QMessageBox::Retry,
+                QMessageBox::Retry);
 }
 
 // taken from utils/fileutils.cpp. We can not use utils here since that depends app_version.h.
@@ -206,48 +211,58 @@ static bool copyRecursively(const QString &srcFilePath,
 
 static inline QStringList getPluginPaths()
 {
-    QStringList rc;
-    // Figure out root:  Up one from 'bin'
-    QDir rootDir = QApplication::applicationDirPath();
-    rootDir.cdUp();
-    const QString rootDirPath = rootDir.canonicalPath();
-    QString pluginPath;
-    if (Utils::HostOsInfo::isMacHost()) {
-        // 1) "PlugIns" (OS X)
-        pluginPath = rootDirPath + QLatin1String("/PlugIns");
-        rc.push_back(pluginPath);
-    } else {
-        // 2) "plugins" (Win/Linux)
-        pluginPath = rootDirPath;
-        pluginPath += QLatin1Char('/');
-        pluginPath += QLatin1String(IDE_LIBRARY_BASENAME);
-        pluginPath += QLatin1String("/qtcreator/plugins");
-        rc.push_back(pluginPath);
-    }
-    // 3) <localappdata>/plugins/<ideversion>
+    QStringList rc(QDir::cleanPath(QApplication::applicationDirPath()
+                                   + '/' + RELATIVE_PLUGIN_PATH));
+    // Local plugin path: <localappdata>/plugins/<ideversion>
     //    where <localappdata> is e.g.
     //    "%LOCALAPPDATA%\QtProject\qtcreator" on Windows Vista and later
     //    "$XDG_DATA_HOME/data/QtProject/qtcreator" or "~/.local/share/data/QtProject/qtcreator" on Linux
     //    "~/Library/Application Support/QtProject/Qt Creator" on Mac
-    pluginPath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-    pluginPath += QLatin1String("/data");
-#endif
+    QString pluginPath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+    if (Utils::HostOsInfo::isAnyUnixHost() && !Utils::HostOsInfo::isMacHost())
+        pluginPath += QLatin1String("/data");
     pluginPath += QLatin1Char('/')
             + QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR)
             + QLatin1Char('/');
-    pluginPath += QLatin1String(Utils::HostOsInfo::isMacHost() ? "Qt Creator" : "qtcreator");
+    pluginPath += QLatin1String(Utils::HostOsInfo::isMacHost() ?
+                                    Core::Constants::IDE_DISPLAY_NAME :
+                                    Core::Constants::IDE_ID);
     pluginPath += QLatin1String("/plugins/");
     pluginPath += QLatin1String(Core::Constants::IDE_VERSION_LONG);
     rc.push_back(pluginPath);
     return rc;
 }
 
+static void setupInstallSettings(QString &installSettingspath)
+{
+    if (!installSettingspath.isEmpty() && !QFileInfo(installSettingspath).isDir()) {
+        displayHelpText(QString("-installsettingspath needs to be the path where a %1/%2.ini exist.").arg(
+            QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR), QLatin1String(Core::Constants::IDE_CASED_ID)));
+        installSettingspath.clear();
+    }
+    // Check if the default install settings contain a setting for the actual install settings.
+    // This can be an absolute path, or a path relative to applicationDirPath().
+    // The result is interpreted like -settingspath, but for SystemScope
+    static const char kInstallSettingsKey[] = "Settings/InstallSettings";
+    QSettings::setPath(QSettings::IniFormat, QSettings::SystemScope,
+        installSettingspath.isEmpty() ? resourcePath() : installSettingspath);
+
+    QSettings installSettings(QSettings::IniFormat, QSettings::UserScope,
+                              QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+                              QLatin1String(Core::Constants::IDE_CASED_ID));
+    if (installSettings.contains(kInstallSettingsKey)) {
+        QString installSettingsPath = installSettings.value(kInstallSettingsKey).toString();
+        if (QDir::isRelativePath(installSettingsPath))
+            installSettingsPath = applicationDirPath() + '/' + installSettingsPath;
+        QSettings::setPath(QSettings::IniFormat, QSettings::SystemScope, installSettingsPath);
+    }
+}
+
 static QSettings *createUserSettings()
 {
     return new QSettings(QSettings::IniFormat, QSettings::UserScope,
                          QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
-                         QLatin1String("QtCreator"));
+                         QLatin1String(Core::Constants::IDE_CASED_ID));
 }
 
 static inline QSettings *userSettings()
@@ -282,9 +297,9 @@ static inline QSettings *userSettings()
                 || lowerFile.startsWith(QLatin1String("qtversion.xml"))
                 || lowerFile.startsWith(QLatin1String("devices.xml"))
                 || lowerFile.startsWith(QLatin1String("debuggers.xml"))
-                || lowerFile.startsWith(QLatin1String("qtcreator.")))
+                || lowerFile.startsWith(QLatin1String(Core::Constants::IDE_ID) + "."))
             QFile::copy(srcDir.absoluteFilePath(file), destDir.absoluteFilePath(file));
-        if (file == QLatin1String("qtcreator"))
+        if (file == QLatin1String(Core::Constants::IDE_ID))
             copyRecursively(srcDir.absoluteFilePath(file), destDir.absoluteFilePath(file));
     }
 
@@ -293,23 +308,89 @@ static inline QSettings *userSettings()
     return createUserSettings();
 }
 
-static const char *SHARE_PATH =
-        Utils::HostOsInfo::isMacHost() ? "/../Resources" : "/../share/qtcreator";
+static void setHighDpiEnvironmentVariable()
+{
+
+    if (Utils::HostOsInfo().isMacHost())
+        return;
+
+    std::unique_ptr<QSettings> settings(createUserSettings());
+
+    const bool defaultValue = Utils::HostOsInfo().isWindowsHost();
+    const bool enableHighDpiScaling = settings->value("Core/EnableHighDpiScaling", defaultValue).toBool();
+
+    static const char ENV_VAR_QT_DEVICE_PIXEL_RATIO[] = "QT_DEVICE_PIXEL_RATIO";
+    if (enableHighDpiScaling
+            && !qEnvironmentVariableIsSet(ENV_VAR_QT_DEVICE_PIXEL_RATIO) // legacy in 5.6, but still functional
+            && !qEnvironmentVariableIsSet("QT_AUTO_SCREEN_SCALE_FACTOR")
+            && !qEnvironmentVariableIsSet("QT_SCALE_FACTOR")
+            && !qEnvironmentVariableIsSet("QT_SCREEN_SCALE_FACTORS")) {
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+    }
+}
 
 void loadFonts()
 {
-    const QDir dir(QCoreApplication::applicationDirPath() + QLatin1String(SHARE_PATH)
-                   + QLatin1String("/fonts/"));
+    const QDir dir(resourcePath() + "/fonts/");
 
     foreach (const QFileInfo &fileInfo, dir.entryInfoList(QStringList("*.ttf"), QDir::Files))
         QFontDatabase::addApplicationFont(fileInfo.absoluteFilePath());
 }
 
+struct Options
+{
+    QString settingsPath;
+    QString installSettingsPath;
+    QStringList customPluginPaths;
+    std::vector<char *> appArguments;
+    bool hasTestOption = false;
+};
+
+Options parseCommandLine(int argc, char *argv[])
+{
+    Options options;
+    auto it = argv;
+    const auto end = argv + argc;
+    while (it != end) {
+        const auto arg = QString::fromLocal8Bit(*it);
+        const bool hasNext = it + 1 != end;
+        const auto nextArg = hasNext ? QString::fromLocal8Bit(*(it + 1)) : QString();
+
+        if (arg == SETTINGS_OPTION && hasNext) {
+            ++it;
+            options.settingsPath = QDir::fromNativeSeparators(nextArg);
+        } else if (arg == INSTALL_SETTINGS_OPTION && hasNext) {
+            ++it;
+            options.installSettingsPath = QDir::fromNativeSeparators(nextArg);
+        } else if (arg == PLUGINPATH_OPTION && hasNext) {
+            ++it;
+            options.customPluginPaths += QDir::fromNativeSeparators(nextArg);
+        } else { // arguments that are still passed on to the application
+            if (arg == TEST_OPTION)
+                options.hasTestOption = true;
+            options.appArguments.push_back(*it);
+        }
+        ++it;
+    }
+    return options;
+}
+
 int main(int argc, char **argv)
 {
-    const char *highDpiEnvironmentVariable = setHighDpiEnvironmentVariable();
+#ifdef Q_OS_WIN
+    if (!qEnvironmentVariableIsSet("QT_OPENGL"))
+        QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
+#endif
+
+    if (qEnvironmentVariableIsSet("QTCREATOR_DISABLE_NATIVE_MENUBAR")
+            || qgetenv("XDG_CURRENT_DESKTOP").startsWith("Unity")) {
+        QApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
+    }
+
+    Utils::TemporaryDirectory::setMasterTemporaryDirectory(QDir::tempPath() + "/" + Core::Constants::IDE_CASED_ID + "-XXXXXX");
+
+    QLoggingCategory::setFilterRules(QLatin1String("qtc.*.debug=false\nqtc.*.info=false"));
 
-    QLoggingCategory::setFilterRules(QLatin1String("qtc.*.debug=false"));
 #ifdef Q_OS_MAC
     // increase the number of file that can be opened in Qt Creator.
     struct rlimit rl;
@@ -319,13 +400,46 @@ int main(int argc, char **argv)
     setrlimit(RLIMIT_NOFILE, &rl);
 #endif
 
-    SharedTools::QtSingleApplication app((QLatin1String(appNameC)), argc, argv);
+    // Manually determine various command line options
+    // We can't use the regular way of the plugin manager,
+    // because settings can change the way plugin manager behaves
+    Options options = parseCommandLine(argc, argv);
+    applicationDirPath(argv[0]);
 
+    QScopedPointer<Utils::TemporaryDirectory> temporaryCleanSettingsDir;
+    if (options.settingsPath.isEmpty() && options.hasTestOption) {
+        temporaryCleanSettingsDir.reset(new Utils::TemporaryDirectory("qtc-test-settings"));
+        if (!temporaryCleanSettingsDir->isValid())
+            return 1;
+        options.settingsPath = temporaryCleanSettingsDir->path();
+    }
+    if (!options.settingsPath.isEmpty())
+        QSettings::setPath(QSettings::IniFormat, QSettings::UserScope, options.settingsPath);
+
+    // Must be done before any QSettings class is created
+    QSettings::setDefaultFormat(QSettings::IniFormat);
+    setupInstallSettings(options.installSettingsPath);
+    // plugin manager takes control of this settings object
+
+    setHighDpiEnvironmentVariable();
+
+    SharedTools::QtSingleApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
+
+    int numberofArguments = static_cast<int>(options.appArguments.size());
+
+    SharedTools::QtSingleApplication app((QLatin1String(Core::Constants::IDE_DISPLAY_NAME)),
+                                         numberofArguments,
+                                         options.appArguments.data());
+    const QStringList pluginArguments = app.arguments();
+
+    /*Initialize global settings and resetup install settings with QApplication::applicationDirPath */
+    setupInstallSettings(options.installSettingsPath);
+    QSettings *settings = userSettings();
+    QSettings *globalSettings = new QSettings(QSettings::IniFormat, QSettings::SystemScope,
+                                              QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+                                              QLatin1String(Core::Constants::IDE_CASED_ID));
     loadFonts();
 
-    if (highDpiEnvironmentVariable)
-        qunsetenv(highDpiEnvironmentVariable);
-
     if (Utils::HostOsInfo().isWindowsHost()
             && !qFuzzyCompare(qApp->devicePixelRatio(), 1.0)
             && QApplication::style()->objectName().startsWith(
@@ -335,62 +449,18 @@ int main(int argc, char **argv)
     const int threadCount = QThreadPool::globalInstance()->maxThreadCount();
     QThreadPool::globalInstance()->setMaxThreadCount(qMax(4, 2 * threadCount));
 
-    CrashHandlerSetup setupCrashHandler; // Display a backtrace once a serious signal is delivered.
-
+    const QString libexecPath = QCoreApplication::applicationDirPath()
+            + '/' + RELATIVE_LIBEXEC_PATH;
 #ifdef ENABLE_QT_BREAKPAD
-    QtSystemExceptionHandler systemExceptionHandler;
+    QtSystemExceptionHandler systemExceptionHandler(libexecPath);
+#else
+    // Display a backtrace once a serious signal is delivered (Linux only).
+    CrashHandlerSetup setupCrashHandler(Core::Constants::IDE_DISPLAY_NAME,
+                                        CrashHandlerSetup::EnableRestart, libexecPath);
 #endif
 
     app.setAttribute(Qt::AA_UseHighDpiPixmaps);
 
-    // Manually determine -settingspath command line option
-    // We can't use the regular way of the plugin manager, because that needs to parse plugin meta data
-    // but the settings path can influence which plugins are enabled
-    QString settingsPath;
-    QStringList customPluginPaths;
-    QStringList arguments = app.arguments(); // adapted arguments list is passed to plugin manager later
-    QMutableStringListIterator it(arguments);
-    bool testOptionProvided = false;
-    while (it.hasNext()) {
-        const QString &arg = it.next();
-        if (arg == QLatin1String(SETTINGS_OPTION)) {
-            it.remove();
-            if (it.hasNext()) {
-                settingsPath = QDir::fromNativeSeparators(it.next());
-                it.remove();
-            }
-        } else if (arg == QLatin1String(PLUGINPATH_OPTION)) {
-            it.remove();
-            if (it.hasNext()) {
-                customPluginPaths << QDir::fromNativeSeparators(it.next());
-                it.remove();
-            }
-        } else if (arg == QLatin1String(TEST_OPTION)) {
-            testOptionProvided = true;
-        }
-    }
-    QScopedPointer<QTemporaryDir> temporaryCleanSettingsDir;
-    if (settingsPath.isEmpty() && testOptionProvided) {
-        const QString settingsPathTemplate = QDir::cleanPath(QDir::tempPath()
-            + QString::fromLatin1("/qtc-test-settings-XXXXXX"));
-        temporaryCleanSettingsDir.reset(new QTemporaryDir(settingsPathTemplate));
-        if (!temporaryCleanSettingsDir->isValid())
-            return 1;
-        settingsPath = temporaryCleanSettingsDir->path();
-    }
-    if (!settingsPath.isEmpty())
-        QSettings::setPath(QSettings::IniFormat, QSettings::UserScope, settingsPath);
-
-    // Must be done before any QSettings class is created
-    QSettings::setPath(QSettings::IniFormat, QSettings::SystemScope,
-                       QCoreApplication::applicationDirPath() + QLatin1String(SHARE_PATH));
-    QSettings::setDefaultFormat(QSettings::IniFormat);
-    // plugin manager takes control of this settings object
-    QSettings *settings = userSettings();
-
-    QSettings *globalSettings = new QSettings(QSettings::IniFormat, QSettings::SystemScope,
-                                              QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
-                                              QLatin1String("QtCreator"));
     PluginManager pluginManager;
     PluginManager::setPluginIID(QLatin1String("org.qt-project.Qt.QtCreatorPlugin"));
     PluginManager::setGlobalSettings(globalSettings);
@@ -403,11 +473,10 @@ int main(int argc, char **argv)
     QString overrideLanguage = settings->value(QLatin1String("General/OverrideLanguage")).toString();
     if (!overrideLanguage.isEmpty())
         uiLanguages.prepend(overrideLanguage);
-    const QString &creatorTrPath = QCoreApplication::applicationDirPath()
-            + QLatin1String(SHARE_PATH) + QLatin1String("/translations");
+    const QString &creatorTrPath = resourcePath() + "/translations";
     foreach (QString locale, uiLanguages) {
         locale = QLocale(locale).name();
-        if (translator.load(QLatin1String("qtcreator_") + locale, creatorTrPath)) {
+        if (translator.load("qtcreator_" + locale, creatorTrPath)) {
             const QString &qtTrPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
             const QString &qtTrFile = QLatin1String("qt_") + locale;
             // Binary installer puts Qt tr files into creatorTrPath
@@ -431,10 +500,10 @@ int main(int argc, char **argv)
     QNetworkProxyFactory::setUseSystemConfiguration(true);
 
     // Load
-    const QStringList pluginPaths = getPluginPaths() + customPluginPaths;
+    const QStringList pluginPaths = getPluginPaths() + options.customPluginPaths;
     PluginManager::setPluginPaths(pluginPaths);
     QMap<QString, QString> foundAppOptions;
-    if (arguments.size() > 1) {
+    if (pluginArguments.size() > 1) {
         QMap<QString, bool> appOptions;
         appOptions.insert(QLatin1String(HELP_OPTION1), false);
         appOptions.insert(QLatin1String(HELP_OPTION2), false);
@@ -445,7 +514,7 @@ int main(int argc, char **argv)
         appOptions.insert(QLatin1String(PID_OPTION), true);
         appOptions.insert(QLatin1String(BLOCK_OPTION), false);
         QString errorMessage;
-        if (!PluginManager::parseOptions(arguments, appOptions, &foundAppOptions, &errorMessage)) {
+        if (!PluginManager::parseOptions(pluginArguments, appOptions, &foundAppOptions, &errorMessage)) {
             displayError(errorMessage);
             printHelp(QFileInfo(app.applicationFilePath()).baseName());
             return -1;
