@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -60,11 +55,13 @@
 
 namespace Utils {
 
-struct ProjectIntroPagePrivate
+class ProjectIntroPagePrivate
 {
+public:
     ProjectIntroPagePrivate();
     Ui::ProjectIntroPage m_ui;
     bool m_complete;
+    QRegularExpressionValidator m_projectNameValidator;
     // Status label style sheets
     const QString m_errorStyleSheet;
     const QString m_warningStyleSheet;
@@ -88,10 +85,10 @@ ProjectIntroPage::ProjectIntroPage(QWidget *parent) :
 {
     d->m_ui.setupUi(this);
     hideStatusLabel();
-    d->m_ui.nameLineEdit->setInitialText(tr("<Enter_Name>"));
+    d->m_ui.nameLineEdit->setPlaceholderText(tr("Enter project name"));
     d->m_ui.nameLineEdit->setFocus();
     d->m_ui.nameLineEdit->setValidationFunction([this](FancyLineEdit *edit, QString *errorString) {
-        return ProjectIntroPage::validateProjectName(edit->text(), errorString);
+        return validateProjectName(edit->text(), errorString);
     });
     d->m_ui.projectLabel->setVisible(d->m_forceSubProject);
     d->m_ui.projectComboBox->setVisible(d->m_forceSubProject);
@@ -140,6 +137,12 @@ void ProjectIntroPage::setPath(const QString &path)
     d->m_ui.pathChooser->setPath(path);
 }
 
+void ProjectIntroPage::setProjectNameRegularExpression(const QRegularExpression &regEx)
+{
+    Q_ASSERT_X(regEx.isValid(), Q_FUNC_INFO, qPrintable(regEx.errorString()));
+    d->m_projectNameValidator.setRegularExpression(regEx);
+}
+
 void ProjectIntroPage::setProjectName(const QString &name)
 {
     d->m_ui.nameLineEdit->setText(name);
@@ -175,13 +178,13 @@ bool ProjectIntroPage::validate()
         return false;
     }
 
-    // Name valid? Ignore 'DisplayingInitialText' state.
+    // Name valid? Ignore 'DisplayingPlaceholderText' state.
     bool nameValid = false;
     switch (d->m_ui.nameLineEdit->state()) {
     case FancyLineEdit::Invalid:
         displayStatusMessage(Error, d->m_ui.nameLineEdit->errorMessage());
         return false;
-    case FancyLineEdit::DisplayingInitialText:
+    case FancyLineEdit::DisplayingPlaceholderText:
         break;
     case FancyLineEdit::Valid:
         nameValid = true;
@@ -250,23 +253,45 @@ int ProjectIntroPage::projectIndex() const
     return d->m_ui.projectComboBox->currentIndex();
 }
 
-bool ProjectIntroPage::validateProjectName(const QString &name, QString *errorMessage /* = 0*/)
+bool ProjectIntroPage::validateProjectName(const QString &name, QString *errorMessage)
 {
-    // Validation is file name + checking for dots
-    if (!FileNameValidatingLineEdit::validateFileName(name, false, errorMessage))
-        return false;
+    int pos = -1;
+    // if we have a pattern it was set
+    if (!d->m_projectNameValidator.regularExpression().pattern().isEmpty()) {
+        if (name.isEmpty()) {
+            if (errorMessage)
+                *errorMessage = tr("Name is empty.");
+            return false;
+        }
+        // pos is set by reference
+        QString tmp = name;
+        QValidator::State validatorState = d->m_projectNameValidator.validate(tmp, pos);
 
-    int pos = FileUtils::indexOfQmakeUnfriendly(name);
+        // if pos is set by validate it is cought at the bottom where it shows
+        // a more detailed error message
+        if (validatorState != QValidator::Acceptable && (pos == -1 || pos >= name.count())) {
+            if (errorMessage) {
+                *errorMessage = tr("Name does not match \"%1\".").arg(
+                    d->m_projectNameValidator.regularExpression().pattern());
+            }
+            return false;
+        }
+    } else { // no validator means usually a qmake project
+        // Validation is file name + checking for dots
+        if (!FileNameValidatingLineEdit::validateFileName(name, false, errorMessage))
+            return false;
+        if (name.contains(QLatin1Char('.'))) {
+            if (errorMessage)
+                *errorMessage = tr("Invalid character \".\".");
+            return false;
+        }
+        pos = FileUtils::indexOfQmakeUnfriendly(name);
+    }
     if (pos >= 0) {
         if (errorMessage)
             *errorMessage = tr("Invalid character \"%1\" found.").arg(name.at(pos));
         return false;
     }
-    if (name.contains(QLatin1Char('.'))) {
-        if (errorMessage)
-            *errorMessage = tr("Invalid character \".\".");
-        return false;
-    }
     return true;
 }
 
