@@ -29,6 +29,7 @@
 
 #include <QPair>
 #include <QObject>
+#include <QVersionNumber>
 
 namespace ProjectExplorer {
 class Kit;
@@ -48,7 +49,12 @@ class ANDROID_EXPORT AndroidManager : public QObject
 public:
     static QString packageName(ProjectExplorer::Target *target);
     static QString packageName(const Utils::FileName &manifestFile);
-
+    static bool packageInstalled(const QString &deviceSerial,
+                                 const QString &packageName);
+    static void apkInfo(const Utils::FileName &apkPath,
+                        QString *packageName = nullptr,
+                        QVersionNumber *version = nullptr,
+                        QString *activityPath = nullptr);
     static QString intentName(ProjectExplorer::Target *target);
     static QString activityName(ProjectExplorer::Target *target);
 
@@ -56,6 +62,9 @@ public:
     static QString deviceSerialNumber(ProjectExplorer::Target *target);
     static void setDeviceSerialNumber(ProjectExplorer::Target *target, const QString &deviceSerialNumber);
 
+    static int deviceApiLevel(ProjectExplorer::Target *target);
+    static void setDeviceApiLevel(ProjectExplorer::Target *target, int level);
+
     static QString buildTargetSDK(ProjectExplorer::Target *target);
 
     static bool signPackage(ProjectExplorer::Target *target);
@@ -84,6 +93,10 @@ public:
     static AndroidQtSupport *androidQtSupport(ProjectExplorer::Target *target);
     static bool updateGradleProperties(ProjectExplorer::Target *target);
     static int findApiLevel(const Utils::FileName &platformPath);
+
+    static void runAdbCommandDetached(const QStringList &args);
+    static bool runAdbCommand(const QStringList &args, QString *output = nullptr);
+    static bool runAaptCommand(const QStringList &args, QString *output = nullptr);
 };
 
 } // namespace Android
