@@ -27,14 +27,16 @@
 #include "quicktesttreeitem.h"
 #include "quicktestvisitors.h"
 #include "quicktest_utils.h"
-#include "../autotest_utils.h"
 #include "../testcodeparser.h"
 
+#include <cpptools/cppmodelmanager.h>
+#include <cpptools/projectpart.h>
 #include <projectexplorer/session.h>
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/qmljsdialect.h>
 #include <qmljstools/qmljsmodelmanager.h>
 #include <utils/hostosinfo.h>
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 namespace Autotest {
@@ -87,25 +89,27 @@ static bool includesQtQuickTest(const CPlusPlus::Document::Ptr &doc,
 static QString quickTestSrcDir(const CppTools::CppModelManager *cppMM,
                                const QString &fileName)
 {
-    static const QByteArray qtsd(" QUICK_TEST_SOURCE_DIR ");
     const QList<CppTools::ProjectPart::Ptr> parts = cppMM->projectPart(fileName);
     if (parts.size() > 0) {
-        QByteArray projDefines(parts.at(0)->projectDefines);
-        for (const QByteArray &line : projDefines.split('\n')) {
-            if (line.contains(qtsd)) {
-                QByteArray result = line.mid(line.indexOf(qtsd) + qtsd.length());
-                if (result.startsWith('"'))
-                    result.remove(result.length() - 1, 1).remove(0, 1);
-                if (result.startsWith("\\\""))
-                    result.remove(result.length() - 2, 2).remove(0, 2);
-                return QLatin1String(result);
-            }
-        }
+        const ProjectExplorer::Macros &macros = parts.at(0)->projectMacros;
+        auto found = std::find_if(
+                    macros.begin(),
+                    macros.end(),
+                    [] (const ProjectExplorer::Macro &macro) { return macro.key == "QUICK_TEST_SOURCE_DIR"; });
+       if (found != macros.end())  {
+           QByteArray result = found->value;
+           if (result.startsWith('"'))
+               result.remove(result.length() - 1, 1).remove(0, 1);
+           if (result.startsWith("\\\""))
+               result.remove(result.length() - 2, 2).remove(0, 2);
+           return QLatin1String(result);
+       }
     }
     return QString();
 }
 
-static QString quickTestName(const CPlusPlus::Document::Ptr &doc)
+static QString quickTestName(const CPlusPlus::Document::Ptr &doc,
+                             const CPlusPlus::Snapshot &snapshot)
 {
     const QList<CPlusPlus::Document::MacroUse> macros = doc->macroUses();
 
@@ -119,7 +123,18 @@ static QString quickTestName(const CPlusPlus::Document::Ptr &doc)
                                  .mid(arg.bytesBegin(), arg.bytesEnd() - arg.bytesBegin()));
         }
     }
-    return QString();
+
+    // check for using quick_test_main() directly
+    const QString fileName = doc->fileName();
+    const QByteArray &fileContent = CppParser::getFileContent(fileName);
+    CPlusPlus::Document::Ptr document = snapshot.preprocessedDocument(fileContent, fileName);
+    if (document.isNull())
+        return QString();
+    document->check();
+    CPlusPlus::AST *ast = document->translationUnit()->ast();
+    QuickTestAstVisitor astVisitor(document, snapshot);
+    astVisitor.accept(ast);
+    return astVisitor.testBaseName();
 }
 
 QList<QmlJS::Document::Ptr> QuickTestParser::scanDirectoryForQuickTestQmlFiles(const QString &srcDir) const
@@ -209,10 +224,10 @@ static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr
 
 bool QuickTestParser::handleQtQuickTest(QFutureInterface<TestParseResultPtr> futureInterface,
                                         CPlusPlus::Document::Ptr document,
-                                        const Core::Id &id) const
+                                        const Core::Id &id)
 {
     const CppTools::CppModelManager *modelManager = CppTools::CppModelManager::instance();
-    if (quickTestName(document).isEmpty())
+    if (quickTestName(document, m_cppSnapshot).isEmpty())
         return false;
 
     const QString cppFileName = document->fileName();
@@ -220,7 +235,7 @@ bool QuickTestParser::handleQtQuickTest(QFutureInterface<TestParseResultPtr> fut
     if (ppList.isEmpty()) // happens if shutting down while parsing
         return false;
     const QString &proFile = ppList.at(0)->projectFile;
-
+    m_mainCppFiles.insert(cppFileName, proFile);
     const QString srcDir = quickTestSrcDir(modelManager, cppFileName);
     if (srcDir.isEmpty())
         return false;
@@ -237,6 +252,49 @@ bool QuickTestParser::handleQtQuickTest(QFutureInterface<TestParseResultPtr> fut
     return result;
 }
 
+static QMap<QString, QDateTime> qmlFilesWithMTime(const QString &directory)
+{
+    const QFileInfoList &qmlFiles = QDir(directory).entryInfoList({ "*.qml" },
+                                                                  QDir::Files, QDir::Name);
+    QMap<QString, QDateTime> filesAndDates;
+    for (const QFileInfo &info : qmlFiles)
+        filesAndDates.insert(info.fileName(), info.lastModified());
+    return filesAndDates;
+}
+
+void QuickTestParser::handleDirectoryChanged(const QString &directory)
+{
+    const QMap<QString, QDateTime> &filesAndDates = qmlFilesWithMTime(directory);
+    const QMap<QString, QDateTime> &watched = m_watchedFiles.value(directory);
+    const QStringList &keys = watched.keys();
+    if (filesAndDates.keys() != keys) { // removed or added files
+        m_watchedFiles[directory] = filesAndDates;
+        TestTreeModel::instance()->parser()->emitUpdateTestTree(this);
+    } else { // we might still have different timestamps
+        const bool timestampChanged = Utils::anyOf(keys, [&](const QString &file) {
+            return filesAndDates.value(file) != watched.value(file);
+        });
+        if (timestampChanged) {
+            QmlJS::PathsAndLanguages paths;
+            paths.maybeInsert(Utils::FileName::fromString(directory), QmlJS::Dialect::Qml);
+            QFutureInterface<void> future;
+            QmlJS::ModelManagerInterface *qmlJsMM = QmlJS::ModelManagerInterface::instance();
+            QmlJS::ModelManagerInterface::importScan(future, qmlJsMM->workingCopy(), paths, qmlJsMM,
+                                                     true /*emitDocumentChanges*/,
+                                                     false /*onlyTheLib*/,
+                                                     true /*forceRescan*/ );
+        }
+    }
+}
+
+void QuickTestParser::doUpdateWatchPaths(const QStringList &directories)
+{
+    for (const QString &dir : directories) {
+        m_directoryWatcher.addPath(dir);
+        m_watchedFiles[dir] = qmlFilesWithMTime(dir);
+    }
+}
+
 QuickTestParser::QuickTestParser()
     : CppParser()
 {
@@ -245,22 +303,37 @@ QuickTestParser::QuickTestParser()
         const QStringList &dirs = m_directoryWatcher.directories();
         if (!dirs.isEmpty())
             m_directoryWatcher.removePaths(dirs);
+        m_watchedFiles.clear();
     });
     connect(&m_directoryWatcher, &QFileSystemWatcher::directoryChanged,
-                     [this] { TestTreeModel::instance()->parser()->emitUpdateTestTree(this); });
+            this, &QuickTestParser::handleDirectoryChanged);
     connect(this, &QuickTestParser::updateWatchPaths,
-            &m_directoryWatcher, &QFileSystemWatcher::addPaths, Qt::QueuedConnection);
+            this, &QuickTestParser::doUpdateWatchPaths, Qt::QueuedConnection);
 }
 
 QuickTestParser::~QuickTestParser()
 {
 }
 
-void QuickTestParser::init(const QStringList &filesToParse)
+void QuickTestParser::init(const QStringList &filesToParse, bool fullParse)
 {
     m_qmlSnapshot = QmlJSTools::Internal::ModelManager::instance()->snapshot();
-    m_proFilesForQmlFiles = QuickTestUtils::proFilesForQmlFiles(id(), filesToParse);
-    CppParser::init(filesToParse);
+    if (!fullParse) {
+        // in a full parse we get the correct entry points by the respective main
+        m_proFilesForQmlFiles = QuickTestUtils::proFilesForQmlFiles(id(), filesToParse);
+        // get rid of cached main cpp files that are going to get processed anyhow
+        for (const QString &file : filesToParse) {
+            if (m_mainCppFiles.contains(file)) {
+                m_mainCppFiles.remove(file);
+                if (m_mainCppFiles.isEmpty())
+                    break;
+            }
+        }
+    } else {
+        // get rid of all cached main cpp files
+        m_mainCppFiles.clear();
+    }
+    CppParser::init(filesToParse, fullParse);
 }
 
 void QuickTestParser::release()
@@ -288,5 +361,10 @@ bool QuickTestParser::processDocument(QFutureInterface<TestParseResultPtr> futur
     return handleQtQuickTest(futureInterface, document, id());
 }
 
+QString QuickTestParser::projectFileForMainCppFile(const QString &fileName) const
+{
+    return m_mainCppFiles.contains(fileName) ? m_mainCppFiles.value(fileName) : QString();
+}
+
 } // namespace Internal
 } // namespace Autotest
