@@ -24,130 +24,113 @@
 ****************************************************************************/
 
 #include "locator.h"
+
+#include "directoryfilter.h"
+#include "executefilter.h"
+#include "externaltoolsfilter.h"
+#include "filesystemfilter.h"
+#include "javascriptfilter.h"
 #include "locatorconstants.h"
 #include "locatorfiltersfilter.h"
 #include "locatormanager.h"
+#include "locatorsettingspage.h"
 #include "locatorwidget.h"
 #include "opendocumentsfilter.h"
-#include "filesystemfilter.h"
-#include "locatorsettingspage.h"
-#include "externaltoolsfilter.h"
 
 #include <coreplugin/coreplugin.h>
-#include <coreplugin/statusbarwidget.h>
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/settingsdatabase.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/settingsdatabase.h>
+#include <coreplugin/statusbarmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/editormanager_p.h>
+#include <coreplugin/menubarfilter.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
 #include <utils/mapreduce.h>
 #include <utils/qtcassert.h>
+#include <utils/utilsicons.h>
 
+#include <QAction>
+#include <QFuture>
 #include <QSettings>
 #include <QtPlugin>
-#include <QFuture>
-#include <QAction>
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
 #include "spotlightlocatorfilter.h"
 #endif
 
 namespace Core {
 namespace Internal {
 
+static Locator *m_instance = nullptr;
+
+class LocatorData
+{
+public:
+    LocatorManager m_locatorManager;
+
+#ifdef WITH_JAVASCRIPTFILTER
+    JavaScriptFilter m_javaScriptFilter;
+#endif
+    OpenDocumentsFilter m_openDocumentsFilter;
+    FileSystemFilter m_fileSystemFilter;
+    ExecuteFilter m_executeFilter;
+    ExternalToolsFilter m_externalToolsFilter;
+    LocatorFiltersFilter m_locatorsFiltersFilter;
+    MenuBarFilter m_menubarFilter;
+#ifdef Q_OS_MACOS
+    SpotlightLocatorFilter m_spotlightLocatorFilter;
+#endif
+};
+
 Locator::Locator()
 {
+    m_instance = this;
     m_refreshTimer.setSingleShot(false);
     connect(&m_refreshTimer, &QTimer::timeout, this, [this]() { refresh(); });
 }
 
 Locator::~Locator()
 {
-    m_corePlugin->removeObject(m_openDocumentsFilter);
-    m_corePlugin->removeObject(m_fileSystemFilter);
-    m_corePlugin->removeObject(m_executeFilter);
-    m_corePlugin->removeObject(m_settingsPage);
-    m_corePlugin->removeObject(m_externalToolsFilter);
-    delete m_openDocumentsFilter;
-    delete m_fileSystemFilter;
-    delete m_executeFilter;
     delete m_settingsPage;
-    delete m_externalToolsFilter;
+    delete m_locatorData;
     qDeleteAll(m_customFilters);
 }
 
-void Locator::initialize(CorePlugin *corePlugin, const QStringList &, QString *)
+Locator *Locator::instance()
 {
-    m_corePlugin = corePlugin;
+    return m_instance;
+}
 
+void Locator::initialize()
+{
+    m_locatorData = new LocatorData;
     m_settingsPage = new LocatorSettingsPage(this);
-    m_corePlugin->addObject(m_settingsPage);
 
-    m_locatorWidget = new LocatorWidget(this);
-    m_locatorWidget->setEnabled(false);
-    StatusBarWidget *view = new StatusBarWidget;
-    view->setWidget(m_locatorWidget);
-    view->setContext(Context("LocatorWidget"));
-    view->setPosition(StatusBarWidget::First);
-    m_corePlugin->addAutoReleasedObject(view);
-
-    QAction *action = new QAction(m_locatorWidget->windowIcon(), m_locatorWidget->windowTitle(), this);
+    QAction *action = new QAction(Utils::Icons::ZOOM.icon(), tr("Locate..."), this);
     Command *cmd = ActionManager::registerAction(action, Constants::LOCATE);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+K")));
-    connect(action, &QAction::triggered, this, &Locator::openLocator);
-    connect(cmd, &Command::keySequenceChanged,
-            this, [this, cmd]() { updatePlaceholderText(cmd); });
-    updatePlaceholderText(cmd);
+    connect(action, &QAction::triggered, this, [] {
+        LocatorManager::show(QString());
+    });
 
     ActionContainer *mtools = ActionManager::actionContainer(Constants::M_TOOLS);
     mtools->addAction(cmd);
 
-    m_corePlugin->addObject(new LocatorManager(m_locatorWidget));
-
-    m_openDocumentsFilter = new OpenDocumentsFilter;
-    m_corePlugin->addObject(m_openDocumentsFilter);
-
-    m_fileSystemFilter = new FileSystemFilter(m_locatorWidget);
-    m_corePlugin->addObject(m_fileSystemFilter);
-
-    m_executeFilter = new ExecuteFilter();
-    m_corePlugin->addObject(m_executeFilter);
-
-    m_externalToolsFilter = new ExternalToolsFilter;
-    m_corePlugin->addObject(m_externalToolsFilter);
-
-    m_corePlugin->addAutoReleasedObject(new LocatorFiltersFilter(this, m_locatorWidget));
-#ifdef Q_OS_OSX
-    m_corePlugin->addAutoReleasedObject(new SpotlightLocatorFilter);
-#endif
-
+    auto locatorWidget = LocatorManager::createLocatorInputWidget(ICore::mainWindow());
+    StatusBarManager::addStatusBarWidget(locatorWidget, StatusBarManager::First,
+                                         Context("LocatorWidget"));
     connect(ICore::instance(), &ICore::saveSettingsRequested, this, &Locator::saveSettings);
 }
 
-void Locator::updatePlaceholderText(Command *command)
-{
-    QTC_ASSERT(command, return);
-    if (command->keySequence().isEmpty())
-        m_locatorWidget->setPlaceholderText(tr("Type to locate"));
-    else
-        m_locatorWidget->setPlaceholderText(tr("Type to locate (%1)").arg(
-                                                command->keySequence().toString(QKeySequence::NativeText)));
-}
-
-void Locator::openLocator()
-{
-    m_locatorWidget->show(QString());
-}
-
 void Locator::extensionsInitialized()
 {
-    m_filters = ExtensionSystem::PluginManager::getObjects<ILocatorFilter>();
+    m_filters = ILocatorFilter::allLocatorFilters();
     Utils::sort(m_filters, [](const ILocatorFilter *first, const ILocatorFilter *second) -> bool {
         if (first->priority() != second->priority())
             return first->priority() < second->priority();
@@ -172,23 +155,69 @@ bool Locator::delayedInitialize()
 
 void Locator::loadSettings()
 {
-    QSettings *qs = ICore::settings();
+    SettingsDatabase *settings = ICore::settingsDatabase();
+    settings->beginGroup("QuickOpen");
+    m_refreshTimer.setInterval(settings->value("RefreshInterval", 60).toInt() * 60000);
 
-    // Backwards compatibility to old settings location
-    if (qs->contains(QLatin1String("QuickOpen/FiltersFilter"))) {
-        loadSettingsHelper(qs);
-    } else {
-        SettingsDatabase *settings = ICore::settingsDatabase();
-        loadSettingsHelper(settings);
+    for (ILocatorFilter *filter : qAsConst(m_filters)) {
+        if (settings->contains(filter->id().toString())) {
+            const QByteArray state = settings->value(filter->id().toString()).toByteArray();
+            if (!state.isEmpty())
+                filter->restoreState(state);
+        }
     }
+    settings->beginGroup("CustomFilters");
+    QList<ILocatorFilter *> customFilters;
+    const QStringList keys = settings->childKeys();
+    int count = 0;
+    Id baseId(Constants::CUSTOM_FILTER_BASEID);
+    for (const QString &key : keys) {
+        ILocatorFilter *filter = new DirectoryFilter(baseId.withSuffix(++count));
+        filter->restoreState(settings->value(key).toByteArray());
+        customFilters.append(filter);
+    }
+    setCustomFilters(customFilters);
+    settings->endGroup();
+    settings->endGroup();
 
-    qs->remove(QLatin1String("QuickOpen"));
-
-    m_locatorWidget->updateFilterList();
-    m_locatorWidget->setEnabled(true);
     if (m_refreshTimer.interval() > 0)
         m_refreshTimer.start();
     m_settingsInitialized = true;
+    setFilters(m_filters + customFilters);
+}
+
+void Locator::updateFilterActions()
+{
+    QMap<Id, QAction *> actionCopy = m_filterActionMap;
+    m_filterActionMap.clear();
+    // register new actions, update existent
+    for (ILocatorFilter *filter : qAsConst(m_filters)) {
+        if (filter->shortcutString().isEmpty() || filter->isHidden())
+            continue;
+        Id filterId = filter->id();
+        Id actionId = filter->actionId();
+        QAction *action = 0;
+        if (!actionCopy.contains(filterId)) {
+            // register new action
+            action = new QAction(filter->displayName(), this);
+            Command *cmd = ActionManager::registerAction(action, actionId);
+            cmd->setAttribute(Command::CA_UpdateText);
+            connect(action, &QAction::triggered, this, [filter] {
+                LocatorManager::showFilter(filter);
+            });
+        } else {
+            action = actionCopy.take(filterId);
+            action->setText(filter->displayName());
+        }
+        m_filterActionMap.insert(filterId, action);
+    }
+
+    // unregister actions that are deleted now
+    const auto end = actionCopy.end();
+    for (auto it = actionCopy.begin(); it != end; ++it) {
+        ActionManager::unregisterAction(it.value(), it.key().withPrefix("Locator."));
+        delete it.value();
+    }
 }
 
 void Locator::updateEditorManagerPlaceholderText()
@@ -214,7 +243,7 @@ void Locator::updateEditorManagerPlaceholderText()
           "</body></html>")
          .arg(openCommand->keySequence().toString(QKeySequence::NativeText))
          .arg(locateCommand->keySequence().toString(QKeySequence::NativeText))
-         .arg(m_fileSystemFilter->shortcutString());
+         .arg(m_locatorData->m_fileSystemFilter.shortcutString());
 
     QString classes;
     // not nice, but anyhow
@@ -236,29 +265,29 @@ void Locator::updateEditorManagerPlaceholderText()
     EditorManagerPrivate::setPlaceholderText(placeholderText.arg(classes, methods));
 }
 
-void Locator::saveSettings()
+void Locator::saveSettings() const
 {
-    if (m_settingsInitialized) {
-        SettingsDatabase *s = ICore::settingsDatabase();
-        s->beginTransaction();
-        s->beginGroup(QLatin1String("QuickOpen"));
-        s->remove(QString());
-        s->setValue(QLatin1String("RefreshInterval"), refreshInterval());
-        foreach (ILocatorFilter *filter, m_filters) {
-            if (!m_customFilters.contains(filter))
-                s->setValue(filter->id().toString(), filter->saveState());
-        }
-        s->beginGroup(QLatin1String("CustomFilters"));
-        int i = 0;
-        foreach (ILocatorFilter *filter, m_customFilters) {
-            s->setValue(QLatin1String("directory") + QString::number(i),
-                        filter->saveState());
-            ++i;
-        }
-        s->endGroup();
-        s->endGroup();
-        s->endTransaction();
+    if (!m_settingsInitialized)
+        return;
+
+    SettingsDatabase *s = ICore::settingsDatabase();
+    s->beginTransaction();
+    s->beginGroup("QuickOpen");
+    s->remove(QString());
+    s->setValue("RefreshInterval", refreshInterval());
+    for (ILocatorFilter *filter : m_filters) {
+        if (!m_customFilters.contains(filter))
+            s->setValue(filter->id().toString(), filter->saveState());
     }
+    s->beginGroup("CustomFilters");
+    int i = 0;
+    for (ILocatorFilter *filter : m_customFilters) {
+        s->setValue("directory" + QString::number(i), filter->saveState());
+        ++i;
+    }
+    s->endGroup();
+    s->endGroup();
+    s->endTransaction();
 }
 
 /*!
@@ -266,7 +295,7 @@ void Locator::saveSettings()
 */
 QList<ILocatorFilter *> Locator::filters()
 {
-    return m_filters;
+    return m_instance->m_filters;
 }
 
 /*!
@@ -281,8 +310,9 @@ QList<ILocatorFilter *> Locator::customFilters()
 void Locator::setFilters(QList<ILocatorFilter *> f)
 {
     m_filters = f;
+    updateFilterActions();
     updateEditorManagerPlaceholderText(); // possibly some shortcut changed
-    m_locatorWidget->updateFilterList();
+    emit filtersChanged();
 }
 
 void Locator::setCustomFilters(QList<ILocatorFilter *> filters)
@@ -290,7 +320,7 @@ void Locator::setCustomFilters(QList<ILocatorFilter *> filters)
     m_customFilters = filters;
 }
 
-int Locator::refreshInterval()
+int Locator::refreshInterval() const
 {
     return m_refreshTimer.interval() / 60000;
 }
