@@ -23,14 +23,17 @@
 **
 ****************************************************************************/
 
+#include "cppeditor.h"
+#include "cppeditorwidget.h"
 #include "cppeditorplugin.h"
 #include "cppeditortestcase.h"
-#include "cppelementevaluator.h"
-#include "cppfollowsymbolundercursor.h"
-#include "cppvirtualfunctionassistprovider.h"
-#include "cppvirtualfunctionproposalitem.h"
 
+#include <cpptools/cppelementevaluator.h>
+#include <cpptools/cppfollowsymbolundercursor.h>
+#include <cpptools/cppvirtualfunctionassistprovider.h>
+#include <cpptools/cppvirtualfunctionproposalitem.h>
 #include <cpptools/cpptoolstestcase.h>
+#include <cpptools/cppmodelmanager.h>
 
 #include <texteditor/codeassist/genericproposalmodel.h>
 #include <texteditor/codeassist/iassistprocessor.h>
@@ -134,23 +137,22 @@ public:
         AssistInterface *assistInterface
                 = m_editorWidget->createAssistInterface(FollowSymbol, ExplicitlyInvoked);
 
-        using CppTools::Tests::IAssistProposalScopedPointer;
-        const IAssistProposalScopedPointer immediateProposal(
+        const QScopedPointer<IAssistProposal> immediateProposal(
             processor->immediateProposal(assistInterface));
-        const IAssistProposalScopedPointer finalProposal(processor->perform(assistInterface));
+        const QScopedPointer<IAssistProposal> finalProposal(processor->perform(assistInterface));
 
         VirtualFunctionAssistProvider::clearParams();
 
-        m_immediateItems = itemList(immediateProposal.d->model());
-        m_finalItems = itemList(finalProposal.d->model());
+        m_immediateItems = itemList(immediateProposal->model());
+        m_finalItems = itemList(finalProposal->model());
 
         return false;
     }
 
-    static OverrideItemList itemList(IAssistProposalModel *imodel)
+    static OverrideItemList itemList(ProposalModelPtr imodel)
     {
         OverrideItemList result;
-        GenericProposalModel *model = dynamic_cast<GenericProposalModel *>(imodel);
+        GenericProposalModelPtr model = imodel.staticCast<GenericProposalModel>();
         if (!model)
             return result;
 
@@ -324,19 +326,35 @@ F2TestCase::F2TestCase(CppEditorAction action,
     switch (action) {
     case FollowSymbolUnderCursorAction: {
         CppEditorWidget *widget = initialTestFile->m_editorWidget;
-        FollowSymbolUnderCursor *delegate = widget->followSymbolUnderCursorDelegate();
-        VirtualFunctionAssistProvider *original = delegate->virtualFunctionAssistProvider();
+        FollowSymbolInterface &delegate = CppModelManager::instance()->followSymbolInterface();
+        auto* builtinFollowSymbol = dynamic_cast<FollowSymbolUnderCursor *>(&delegate);
+        if (!builtinFollowSymbol) {
+            if (filePaths.size() > 1)
+                QSKIP("Clang FollowSymbol does not currently support multiple files (except cpp+header)");
+            const QString curTestName = QLatin1String(QTest::currentTestFunction());
+            if (curTestName == "test_FollowSymbolUnderCursor_QObject_connect"
+                    || curTestName == "test_FollowSymbolUnderCursor_virtualFunctionCall"
+                    || curTestName == "test_FollowSymbolUnderCursor_QTCREATORBUG7903") {
+                QSKIP((curTestName + " is not supported by Clang FollowSymbol").toLatin1());
+            }
+
+            initialTestFile->m_editorWidget->openLinkUnderCursor();
+            break;
+        }
+
+        QSharedPointer<VirtualFunctionAssistProvider> original
+                = builtinFollowSymbol->virtualFunctionAssistProvider();
 
         // Set test provider, run and get results
-        QScopedPointer<VirtualFunctionTestAssistProvider> testProvider(
+        QSharedPointer<VirtualFunctionTestAssistProvider> testProvider(
             new VirtualFunctionTestAssistProvider(widget));
-        delegate->setVirtualFunctionAssistProvider(testProvider.data());
+        builtinFollowSymbol->setVirtualFunctionAssistProvider(testProvider);
         initialTestFile->m_editorWidget->openLinkUnderCursor();
         immediateVirtualSymbolResults = testProvider->m_immediateItems;
         finalVirtualSymbolResults = testProvider->m_finalItems;
 
         // Restore original test provider
-        delegate->setVirtualFunctionAssistProvider(original);
+        builtinFollowSymbol->setVirtualFunctionAssistProvider(original);
         break;
     }
     case SwitchBetweenMethodDeclarationDefinitionAction:
@@ -868,41 +886,6 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
         "@Container<int> container;\n"
     );
 
-    QTest::newRow("using_QTCREATORBUG7903_globalNamespace") << _(
-        "namespace NS {\n"
-        "class Foo {};\n"
-        "}\n"
-        "using NS::$Foo;\n"
-        "void fun()\n"
-        "{\n"
-        "    @Foo foo;\n"
-        "}\n"
-    );
-
-    QTest::newRow("using_QTCREATORBUG7903_namespace") << _(
-        "namespace NS {\n"
-        "class Foo {};\n"
-        "}\n"
-        "namespace NS1 {\n"
-        "void fun()\n"
-        "{\n"
-        "    using NS::$Foo;\n"
-        "    @Foo foo;\n"
-        "}\n"
-        "}\n"
-    );
-
-    QTest::newRow("using_QTCREATORBUG7903_insideFunction") << _(
-        "namespace NS {\n"
-        "class Foo {};\n"
-        "}\n"
-        "void fun()\n"
-        "{\n"
-        "    using NS::$Foo;\n"
-        "    @Foo foo;\n"
-        "}\n"
-    );
-
     QTest::newRow("matchFunctionSignature_Follow_1") << _(
         "class Foo {\n"
         "    void @foo(int);\n"
@@ -941,6 +924,34 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
         "};\n"
     );
 
+    QTest::newRow("matchFunctionSignature_Follow_6") << _(
+        "class Foo {\n"
+        "    void $foo(int);\n"
+        "};\n"
+        "void Foo::@foo(const volatile int) {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_7") << _(
+        "class Foo {\n"
+        "    void $foo(const volatile int);\n"
+        "};\n"
+        "void Foo::@foo(int) {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_8") << _(
+        "class Foo {\n"
+        "    void @$foo(int *);\n"
+        "};\n"
+        "void Foo::foo(const int *) {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_9") << _(
+        "class Foo {\n"
+        "    void @$foo(int&);\n"
+        "};\n"
+        "void Foo::foo(const int&) {}\n"
+    );
+
     QTest::newRow("infiniteLoopLocalTypedef_QTCREATORBUG-11999") << _(
         "template<class MyTree>\n"
         "class TreeConstIterator\n"
@@ -963,6 +974,8 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
     );
 
     QTest::newRow("template_alias") << _(
+        "template<class T>"
+        "class Bar;"
         "template<class $T>\n"
         "using Foo = Bar<@T>;\n"
     );
@@ -974,6 +987,51 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
+void CppEditorPlugin::test_FollowSymbolUnderCursor_QTCREATORBUG7903_data()
+{
+    QTest::addColumn<QByteArray>("source");
+    QTest::newRow("using_QTCREATORBUG7903_globalNamespace") << _(
+        "namespace NS {\n"
+        "class Foo {};\n"
+        "}\n"
+        "using NS::$Foo;\n"
+        "void fun()\n"
+        "{\n"
+        "    @Foo foo;\n"
+        "}\n"
+    );
+
+    QTest::newRow("using_QTCREATORBUG7903_namespace") << _(
+        "namespace NS {\n"
+        "class Foo {};\n"
+        "}\n"
+        "namespace NS1 {\n"
+        "void fun()\n"
+        "{\n"
+        "    using NS::$Foo;\n"
+        "    @Foo foo;\n"
+        "}\n"
+        "}\n"
+    );
+
+    QTest::newRow("using_QTCREATORBUG7903_insideFunction") << _(
+        "namespace NS {\n"
+        "class Foo {};\n"
+        "}\n"
+        "void fun()\n"
+        "{\n"
+        "    using NS::$Foo;\n"
+        "    @Foo foo;\n"
+        "}\n"
+    );
+}
+
+void CppEditorPlugin::test_FollowSymbolUnderCursor_QTCREATORBUG7903()
+{
+    QFETCH(QByteArray, source);
+    F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
+}
+
 void CppEditorPlugin::test_FollowSymbolUnderCursor_followCall_data()
 {
     QTest::addColumn<QByteArray>("variableDeclaration"); // without semicolon, can be ""
