@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,37 +9,35 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "qmlcppengine.h"
 #include "qmlengine.h"
-#include <debugger/debuggerruncontrolfactory.h>
-#include <debugger/debuggerstartparameters.h>
+
+#include <debugger/debuggercore.h>
+#include <debugger/debuggerruncontrol.h>
+#include <debugger/debuggertooltipmanager.h>
+#include <debugger/breakhandler.h>
 #include <debugger/stackhandler.h>
+#include <debugger/threaddata.h>
 #include <debugger/watchhandler.h>
+#include <debugger/console/console.h>
 
 #include <utils/qtcassert.h>
-#include <texteditor/itexteditor.h>
 #include <qmljseditor/qmljseditorconstants.h>
 #include <cppeditor/cppeditorconstants.h>
-#include <qmljs/consolemanagerinterface.h>
 
 namespace Debugger {
 namespace Internal {
@@ -48,14 +46,11 @@ enum { debug = 0 };
 
 #define EDEBUG(s) do { if (debug) qDebug() << s; } while (0)
 
-DebuggerEngine *createQmlCppEngine(const DebuggerStartParameters &sp,
-                                   QString *errorMessage)
+#define CHECK_STATE(s) do { checkState(s, __FILE__, __LINE__); } while (0)
+
+DebuggerEngine *createQmlCppEngine(DebuggerEngine *cppEngine)
 {
-    QmlCppEngine *newEngine = new QmlCppEngine(sp, errorMessage);
-    if (newEngine->cppEngine())
-        return newEngine;
-    delete newEngine;
-    return 0;
+    return new QmlCppEngine(cppEngine);
 }
 
 
@@ -65,18 +60,14 @@ DebuggerEngine *createQmlCppEngine(const DebuggerStartParameters &sp,
 //
 ////////////////////////////////////////////////////////////////////////
 
-QmlCppEngine::QmlCppEngine(const DebuggerStartParameters &sp, QString *errorMessage)
-    : DebuggerEngine(sp)
+QmlCppEngine::QmlCppEngine(DebuggerEngine *cppEngine)
 {
-    setObjectName(QLatin1String("QmlCppEngine"));
-    m_qmlEngine = new QmlEngine(sp, this);
-    m_cppEngine = DebuggerRunControlFactory::createEngine(sp.firstSlaveEngineType, sp, errorMessage);
-    if (!m_cppEngine) {
-        *errorMessage = tr("The slave debugging engine required for combined QML/C++-Debugging could not be created: %1").arg(*errorMessage);
-        return;
-    }
+    setObjectName("QmlCppEngine");
+    m_qmlEngine = new QmlEngine;
+    m_qmlEngine->setMasterEngine(this);
+    m_cppEngine = cppEngine;
     m_cppEngine->setMasterEngine(this);
-    setActiveEngine(m_cppEngine);
+    m_activeEngine = m_cppEngine;
 }
 
 QmlCppEngine::~QmlCppEngine()
@@ -90,31 +81,36 @@ bool QmlCppEngine::canDisplayTooltip() const
     return m_cppEngine->canDisplayTooltip() || m_qmlEngine->canDisplayTooltip();
 }
 
-bool QmlCppEngine::setToolTipExpression(TextEditor::ITextEditor *editor, const DebuggerToolTipContext &ctx)
+bool QmlCppEngine::canHandleToolTip(const DebuggerToolTipContext &ctx) const
 {
-    QTC_ASSERT(editor, return false);
     bool success = false;
-    if (editor->document()->id() == CppEditor::Constants::CPPEDITOR_ID)
-        success = m_cppEngine->setToolTipExpression(editor, ctx);
-    else if (editor->document()->id() == QmlJSEditor::Constants::C_QMLJSEDITOR_ID)
-        success = m_qmlEngine->setToolTipExpression(editor, ctx);
+    if (ctx.isCppEditor)
+        success = m_cppEngine->canHandleToolTip(ctx);
+    else
+        success = m_qmlEngine->canHandleToolTip(ctx);
     return success;
 }
 
-void QmlCppEngine::updateWatchData(const WatchData &data,
-    const WatchUpdateFlags &flags)
+void QmlCppEngine::updateItem(const QString &iname)
 {
-    if (data.isInspect())
-        m_qmlEngine->updateWatchData(data, flags);
+    if (iname.startsWith("inspect."))
+        m_qmlEngine->updateItem(iname);
     else
-        m_activeEngine->updateWatchData(data, flags);
+        m_activeEngine->updateItem(iname);
 }
 
-void QmlCppEngine::watchDataSelected(const QByteArray &iname)
+void QmlCppEngine::expandItem(const QString &iname)
 {
-    const WatchData *wd = watchHandler()->findData(iname);
-    if (wd && wd->isInspect())
-        m_qmlEngine->watchDataSelected(iname);
+    if (iname.startsWith("inspect."))
+        m_qmlEngine->expandItem(iname);
+    else
+        m_activeEngine->expandItem(iname);
+}
+
+void QmlCppEngine::selectWatchData(const QString &iname)
+{
+    if (iname.startsWith("inspect."))
+        m_qmlEngine->selectWatchData(iname);
 }
 
 void QmlCppEngine::watchPoint(const QPoint &point)
@@ -122,10 +118,14 @@ void QmlCppEngine::watchPoint(const QPoint &point)
     m_cppEngine->watchPoint(point);
 }
 
-void QmlCppEngine::fetchMemory(MemoryAgent *ma, QObject *obj,
-        quint64 addr, quint64 length)
+void QmlCppEngine::fetchMemory(MemoryAgent *agent, quint64 addr, quint64 length)
 {
-    m_cppEngine->fetchMemory(ma, obj, addr, length);
+    m_cppEngine->fetchMemory(agent, addr, length);
+}
+
+void QmlCppEngine::changeMemory(MemoryAgent *agent, quint64 addr, const QByteArray &data)
+{
+    m_cppEngine->changeMemory(agent, addr, data);
 }
 
 void QmlCppEngine::fetchDisassembler(DisassemblerAgent *da)
@@ -183,9 +183,9 @@ void QmlCppEngine::reloadFullStack()
     m_cppEngine->reloadFullStack();
 }
 
-void QmlCppEngine::setRegisterValue(int regnr, const QString &value)
+void QmlCppEngine::setRegisterValue(const QString &name, const QString &value)
 {
-    m_cppEngine->setRegisterValue(regnr, value);
+    m_cppEngine->setRegisterValue(name, value);
 }
 
 
@@ -212,7 +212,7 @@ bool QmlCppEngine::isSynchronous() const
     return m_activeEngine->isSynchronous();
 }
 
-QByteArray QmlCppEngine::qtNamespace() const
+QString QmlCppEngine::qtNamespace() const
 {
     return m_cppEngine->qtNamespace();
 }
@@ -243,10 +243,15 @@ void QmlCppEngine::attemptBreakpointSynchronization()
     }
 }
 
-bool QmlCppEngine::acceptsBreakpoint(BreakpointModelId id) const
+void QmlCppEngine::doUpdateLocals(const UpdateParameters &up)
 {
-    return m_cppEngine->acceptsBreakpoint(id)
-        || m_qmlEngine->acceptsBreakpoint(id);
+    m_activeEngine->doUpdateLocals(up);
+}
+
+bool QmlCppEngine::acceptsBreakpoint(Breakpoint bp) const
+{
+    return m_cppEngine->acceptsBreakpoint(bp)
+        || m_qmlEngine->acceptsBreakpoint(bp);
 }
 
 void QmlCppEngine::selectThread(ThreadId threadId)
@@ -254,22 +259,17 @@ void QmlCppEngine::selectThread(ThreadId threadId)
     m_activeEngine->selectThread(threadId);
 }
 
-void QmlCppEngine::assignValueInDebugger(const WatchData *data,
+void QmlCppEngine::assignValueInDebugger(WatchItem *item,
     const QString &expr, const QVariant &value)
 {
-    if (data->isInspect())
-        m_qmlEngine->assignValueInDebugger(data, expr, value);
+    if (item->isInspect())
+        m_qmlEngine->assignValueInDebugger(item, expr, value);
     else
-        m_activeEngine->assignValueInDebugger(data, expr, value);
+        m_activeEngine->assignValueInDebugger(item, expr, value);
 }
 
 void QmlCppEngine::notifyInferiorIll()
 {
-    //This will eventually shutdown the engine
-    //Set final state to avoid quitDebugger() being called
-    //after this call
-    setTargetState(DebuggerFinished);
-
     //Call notifyInferiorIll of cpp engine
     //as qml engine will follow state transitions
     //of cpp engine
@@ -284,26 +284,10 @@ void QmlCppEngine::detachDebugger()
 
 void QmlCppEngine::executeStep()
 {
-//    TODO: stepping from qml -> cpp requires more thought
-//    if (m_activeEngine == m_qmlEngine) {
-//        QTC_CHECK(m_cppEngine->state() == InferiorRunOk);
-//        if (m_cppEngine->setupQmlStep(true))
-//            return; // Wait for callback to readyToExecuteQmlStep()
-//    } else {
-//        notifyInferiorRunRequested();
-//        m_cppEngine->executeStep();
-//    }
-
     notifyInferiorRunRequested();
     m_activeEngine->executeStep();
 }
 
-void QmlCppEngine::readyToExecuteQmlStep()
-{
-    notifyInferiorRunRequested();
-    m_qmlEngine->executeStep();
-}
-
 void QmlCppEngine::executeStepOut()
 {
     notifyInferiorRunRequested();
@@ -353,13 +337,8 @@ void QmlCppEngine::continueInferior()
 void QmlCppEngine::interruptInferior()
 {
     EDEBUG("\nMASTER INTERRUPT INFERIOR");
-    m_cppEngine->requestInterruptInferior();
-}
-
-void QmlCppEngine::requestInterruptInferior()
-{
-    EDEBUG("\nMASTER REQUEST INTERRUPT INFERIOR");
-    DebuggerEngine::requestInterruptInferior();
+    m_activeEngine->setState(InferiorStopRequested);
+    m_activeEngine->interruptInferior();
 }
 
 void QmlCppEngine::executeRunToLine(const ContextData &data)
@@ -388,55 +367,9 @@ void QmlCppEngine::executeDebuggerCommand(const QString &command, DebuggerLangua
 void QmlCppEngine::setupEngine()
 {
     EDEBUG("\nMASTER SETUP ENGINE");
-    setActiveEngine(m_cppEngine);
+    m_activeEngine = m_cppEngine;
     m_qmlEngine->setupSlaveEngine();
     m_cppEngine->setupSlaveEngine();
-
-    if (startParameters().remoteSetupNeeded)
-        notifyEngineRequestRemoteSetup();
-}
-
-void QmlCppEngine::notifyEngineRunAndInferiorRunOk()
-{
-    EDEBUG("\nMASTER NOTIFY ENGINE RUN AND INFERIOR RUN OK");
-    DebuggerEngine::notifyEngineRunAndInferiorRunOk();
-}
-
-void QmlCppEngine::notifyInferiorRunOk()
-{
-    EDEBUG("\nMASTER NOTIFY INFERIOR RUN OK");
-    DebuggerEngine::notifyInferiorRunOk();
-}
-
-void QmlCppEngine::notifyInferiorSpontaneousStop()
-{
-    EDEBUG("\nMASTER SPONTANEOUS STOP OK");
-    DebuggerEngine::notifyInferiorSpontaneousStop();
-}
-
-void QmlCppEngine::notifyInferiorShutdownOk()
-{
-    EDEBUG("\nMASTER INFERIOR SHUTDOWN OK");
-    DebuggerEngine::notifyInferiorShutdownOk();
-}
-
-void QmlCppEngine::notifyInferiorSetupOk()
-{
-    EDEBUG("\nMASTER INFERIOR SETUP OK");
-    emit aboutToNotifyInferiorSetupOk();
-    DebuggerEngine::notifyInferiorSetupOk();
-}
-
-void QmlCppEngine::notifyEngineRemoteServerRunning(const QByteArray &serverChannel, int pid)
-{
-    m_cppEngine->notifyEngineRemoteServerRunning(serverChannel, pid);
-}
-
-void QmlCppEngine::setupInferior()
-{
-    EDEBUG("\nMASTER SETUP INFERIOR");
-    m_qmlEngine->setupSlaveInferior();
-    m_cppEngine->setupSlaveInferior();
 }
 
 void QmlCppEngine::runEngine()
@@ -455,10 +388,8 @@ void QmlCppEngine::shutdownInferior()
 void QmlCppEngine::shutdownEngine()
 {
     EDEBUG("\nMASTER SHUTDOWN ENGINE");
+    m_qmlEngine->shutdownSlaveEngine();
     m_cppEngine->shutdownSlaveEngine();
-    QmlJS::ConsoleManagerInterface *consoleManager = QmlJS::ConsoleManagerInterface::instance();
-    if (consoleManager)
-        consoleManager->setScriptEvaluator(0);
 }
 
 void QmlCppEngine::quitDebugger()
@@ -467,10 +398,10 @@ void QmlCppEngine::quitDebugger()
     m_cppEngine->quitDebugger();
 }
 
-void QmlCppEngine::abortDebugger()
+void QmlCppEngine::abortDebuggerProcess()
 {
     EDEBUG("\nMASTER ABORT DEBUGGER");
-    m_cppEngine->abortDebugger();
+    m_cppEngine->abortDebuggerProcess();
 }
 
 void QmlCppEngine::setState(DebuggerState newState, bool forced)
@@ -485,8 +416,9 @@ void QmlCppEngine::slaveEngineStateChanged
     (DebuggerEngine *slaveEngine, const DebuggerState newState)
 {
     DebuggerEngine *otherEngine = (slaveEngine == m_cppEngine)
-         ? m_qmlEngine : m_cppEngine;
+         ? m_qmlEngine.data() : m_cppEngine.data();
 
+    QTC_ASSERT(otherEngine, return);
     QTC_CHECK(otherEngine != slaveEngine);
 
     if (debug) {
@@ -495,6 +427,11 @@ void QmlCppEngine::slaveEngineStateChanged
         EDEBUG("  COMBINED ENGINE: " << this << state() << isDying());
     }
 
+    if (state() == DebuggerFinished) {
+        // We are done and don't care about slave state changes anymore.
+        return;
+    }
+
     // Idea is to follow the state of the cpp engine, except where we are stepping in QML.
     // That is, when the QmlEngine moves between InferiorStopOk, and InferiorRunOk, InferiorStopOk ...
     //
@@ -509,12 +446,12 @@ void QmlCppEngine::slaveEngineStateChanged
             break;
         }
         case EngineSetupRequested: {
-            // set by queueSetupEngine()
-            QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
+            // Set by queueSetupEngine()
+            CHECK_STATE(EngineSetupRequested);
             break;
         }
         case EngineSetupFailed: {
-            qmlEngine()->quitDebugger();
+            m_qmlEngine->quitDebugger();
             notifyEngineSetupFailed();
             break;
         }
@@ -522,51 +459,41 @@ void QmlCppEngine::slaveEngineStateChanged
             notifyEngineSetupOk();
             break;
         }
-        case InferiorSetupRequested: {
-            // set by queueSetupInferior()
-            QTC_ASSERT(state() == InferiorSetupRequested, qDebug() << state());
-            break;
-        }
-        case InferiorSetupFailed: {
-            qmlEngine()->quitDebugger();
-            notifyInferiorSetupFailed();
-            break;
-        }
-        case InferiorSetupOk: {
-            notifyInferiorSetupOk();
-            break;
-        }
         case EngineRunRequested: {
             // set by queueRunEngine()
             break;
         }
         case EngineRunFailed: {
-            qmlEngine()->quitDebugger();
+            m_qmlEngine->quitDebugger();
             notifyEngineRunFailed();
             break;
         }
         case InferiorUnrunnable: {
-            qmlEngine()->quitDebugger();
-            notifyInferiorUnrunnable();
+            m_qmlEngine->quitDebugger();
+            notifyEngineRunOkAndInferiorUnrunnable();
             break;
         }
         case InferiorRunRequested: {
-            // might be set already by notifyInferiorRunRequested()
-            QTC_ASSERT(state() == InferiorRunRequested
-                       || state() == InferiorStopOk, qDebug() << state());
-            if (state() != InferiorRunRequested)
+            // Might be set already by notifyInferiorRunRequested()
+            if (state() != InferiorRunRequested) {
+                CHECK_STATE(InferiorStopOk);
                 notifyInferiorRunRequested();
+            }
             break;
         }
         case InferiorRunOk: {
-            QTC_ASSERT(state() == EngineRunRequested
-                       || state() == InferiorRunRequested, qDebug() << state());
-            if (state() == EngineRunRequested)
+            if (state() == EngineRunRequested) {
                 notifyEngineRunAndInferiorRunOk();
-            else if (state() == InferiorRunRequested)
+            } else if (state() == InferiorRunRequested) {
                 notifyInferiorRunOk();
+            } else if (state() == InferiorStopOk) {
+                notifyInferiorRunRequested();
+                notifyInferiorRunOk();
+            } else {
+                QTC_ASSERT(false, qDebug() << state());
+            }
 
-            if (qmlEngine()->state() == InferiorStopOk) {
+            if (m_qmlEngine->state() == InferiorStopOk) {
                 // track qml engine again
                 setState(InferiorStopRequested);
                 notifyInferiorStopOk();
@@ -575,32 +502,33 @@ void QmlCppEngine::slaveEngineStateChanged
             break;
         }
         case InferiorRunFailed: {
-            qmlEngine()->quitDebugger();
+            m_qmlEngine->quitDebugger();
             notifyInferiorRunFailed();
             break;
         }
         case InferiorStopRequested: {
             if (m_activeEngine == cppEngine()) {
-                // might be set by doInterruptInferior()
-                QTC_ASSERT(state() == InferiorStopRequested
-                           || state() == InferiorRunOk, qDebug() << state());
-                if (state() == InferiorRunOk)
+                if (state() == InferiorRunOk) {
                     setState(InferiorStopRequested);
+                } else {
+                    // Might be set by doInterruptInferior()
+                    CHECK_STATE(InferiorStopRequested);
+                }
             } else {
-                // we're debugging qml, but got an interrupt, or abort
-                QTC_ASSERT(state() == InferiorRunOk
-                          || state() == InferiorStopOk
-                           || state() == InferiorRunRequested, qDebug() << state());
-
+                // We're debugging qml, but got an interrupt, or abort
                 if (state() == InferiorRunOk) {
                     setState(InferiorStopRequested);
                 } else if (state() == InferiorStopOk) {
-                    notifyInferiorRunRequested();
-                    notifyInferiorRunOk();
-                    setState(InferiorStopRequested);
+                    if (!isDying()) {
+                        notifyInferiorRunRequested();
+                        notifyInferiorRunOk();
+                        setState(InferiorStopRequested);
+                    }
                 } else if (state() == InferiorRunRequested) {
                     notifyInferiorRunOk();
                     setState(InferiorStopRequested);
+                } else {
+                    QTC_ASSERT(false, qDebug() << state());
                 }
                 // now track cpp engine
                 setActiveEngine(m_cppEngine);
@@ -615,7 +543,7 @@ void QmlCppEngine::slaveEngineStateChanged
                            || state() == InferiorStopOk, qDebug() << state());
 
                 // Just to make sure, we're shutting down anyway ...
-                setActiveEngine(m_cppEngine);
+                m_activeEngine = m_cppEngine;
 
                 if (state() == InferiorStopRequested)
                     setState(InferiorStopOk);
@@ -625,12 +553,6 @@ void QmlCppEngine::slaveEngineStateChanged
                     showStatusMessage(tr("C++ debugger activated"));
                     setActiveEngine(m_cppEngine);
                 }
-
-                QTC_ASSERT(state() == InferiorStopRequested
-                           || state() == InferiorRunRequested
-                           || state() == EngineRunRequested
-                           || state() == InferiorRunOk
-                           || state() == InferiorStopOk, qDebug() << state());
                 switch (state()) {
                 case InferiorStopRequested:
                     EDEBUG("... CPP ENGINE STOPPED EXPECTEDLY");
@@ -647,40 +569,32 @@ void QmlCppEngine::slaveEngineStateChanged
                 case InferiorRunRequested:
                     // can happen if qml engine was active
                     notifyInferiorRunFailed();
+                    break;
                 default:
+                    CHECK_STATE(InferiorStopOk);
                     break;
                 }
             }
             break;
         }
         case InferiorStopFailed: {
-            QTC_ASSERT(state() == InferiorStopRequested, qDebug() << state());
+            CHECK_STATE(InferiorStopRequested);
             notifyInferiorStopFailed();
             break;
         }
-        case InferiorExitOk: {
-            // InferiorExitOk will be called through notifyInferiorExited
-            // when InferiorShutDownOk is reached
-            qmlEngine()->quitDebugger();
-            break;
-        }
         case InferiorShutdownRequested: {
-            // might be set by queueShutdownInferior() already
-            QTC_ASSERT(state() == InferiorShutdownRequested
-                       || state() == InferiorStopOk, qDebug() << state());
-            if (state() == InferiorStopOk)
+            if (state() == InferiorStopOk) {
                 setState(InferiorShutdownRequested);
-            qmlEngine()->quitDebugger();
+            } else {
+                // might be set by queueShutdownInferior() already
+                CHECK_STATE(InferiorShutdownRequested);
+            }
+            m_qmlEngine->quitDebugger();
             break;
         }
-        case InferiorShutdownFailed: {
-            QTC_ASSERT(state() == InferiorShutdownRequested, qDebug() << state());
-            notifyInferiorShutdownFailed();
-            break;
-        }
-        case InferiorShutdownOk: {
+        case InferiorShutdownFinished: {
             if (state() == InferiorShutdownRequested) {
-                notifyInferiorShutdownOk();
+                notifyInferiorShutdownFinished();
             } else {
                 // we got InferiorExitOk before, but ignored it ...
                 notifyInferiorExited();
@@ -689,22 +603,17 @@ void QmlCppEngine::slaveEngineStateChanged
         }
         case EngineShutdownRequested: {
             // set by queueShutdownEngine()
-            QTC_ASSERT(state() == EngineShutdownRequested, qDebug() << state());
+            CHECK_STATE(EngineShutdownRequested);
             break;
         }
-        case EngineShutdownFailed: {
-            QTC_ASSERT(state() == EngineShutdownRequested, qDebug() << state());
-            notifyEngineShutdownFailed();
-            break;
-        }
-        case EngineShutdownOk: {
-            QTC_ASSERT(state() == EngineShutdownRequested, qDebug() << state());
-            notifyEngineShutdownOk();
+        case EngineShutdownFinished: {
+            CHECK_STATE(EngineShutdownRequested);
+            notifyEngineShutdownFinished();
             break;
         }
         case DebuggerFinished: {
             // set by queueFinishDebugger()
-            QTC_ASSERT(state() == DebuggerFinished, qDebug() << state());
+            CHECK_STATE(DebuggerFinished);
             break;
         }
         }
@@ -721,57 +630,28 @@ void QmlCppEngine::slaveEngineStateChanged
                     notifyInferiorStopOk();
                 // otherwise we're probably inside notifyInferiorStopOk already
             } else {
-                if (m_activeEngine != qmlEngine()) {
+                if (m_activeEngine != m_qmlEngine) {
                     showStatusMessage(tr("QML debugger activated"));
                     setActiveEngine(m_qmlEngine);
                 }
 
-                QTC_ASSERT(state() == InferiorRunOk
-                           || state() == InferiorStopRequested
-                           || state() == InferiorShutdownRequested, qDebug() << state());
-
                 if (state() == InferiorRunOk)
                     notifyInferiorSpontaneousStop();
                 else if (state() == InferiorStopRequested)
                     notifyInferiorStopOk();
+                else
+                    CHECK_STATE(InferiorShutdownRequested);
             }
 
         } else if (newState == InferiorRunOk) {
-            if (m_activeEngine == qmlEngine()) {
-                QTC_ASSERT(state() == InferiorRunRequested, qDebug() << state());
+            if (m_activeEngine == m_qmlEngine) {
+                CHECK_STATE(InferiorRunRequested);
                 notifyInferiorRunOk();
             }
         }
     }
 }
 
-void QmlCppEngine::notifyEngineRemoteSetupDone(int gdbServerPort, int qmlPort)
-{
-    EDEBUG("MASTER REMOTE SETUP DONE");
-    DebuggerEngine::notifyEngineRemoteSetupDone(gdbServerPort, qmlPort);
-
-    cppEngine()->notifyEngineRemoteSetupDone(gdbServerPort, qmlPort);
-    qmlEngine()->notifyEngineRemoteSetupDone(gdbServerPort, qmlPort);
-}
-
-void QmlCppEngine::notifyEngineRemoteSetupFailed(const QString &message)
-{
-    EDEBUG("MASTER REMOTE SETUP FAILED");
-    DebuggerEngine::notifyEngineRemoteSetupFailed(message);
-
-    cppEngine()->notifyEngineRemoteSetupFailed(message);
-    qmlEngine()->notifyEngineRemoteSetupFailed(message);
-}
-
-void QmlCppEngine::showMessage(const QString &msg, int channel, int timeout) const
-{
-    if (channel == AppOutput || channel == AppError || channel == AppStuff) {
-        // message is from CppEngine, allow qml engine to process
-        m_qmlEngine->filterApplicationMessage(msg, channel);
-    }
-    DebuggerEngine::showMessage(msg, channel, timeout);
-}
-
 void QmlCppEngine::resetLocation()
 {
     if (m_qmlEngine)
@@ -782,14 +662,23 @@ void QmlCppEngine::resetLocation()
     DebuggerEngine::resetLocation();
 }
 
-DebuggerEngine *QmlCppEngine::cppEngine() const
+void QmlCppEngine::reloadDebuggingHelpers()
 {
-    return m_cppEngine;
+    if (m_cppEngine)
+        m_cppEngine->reloadDebuggingHelpers();
 }
 
-DebuggerEngine *QmlCppEngine::qmlEngine() const
+void QmlCppEngine::debugLastCommand()
 {
-    return m_qmlEngine;
+    if (m_cppEngine)
+        m_cppEngine->debugLastCommand();
+}
+
+void QmlCppEngine::setRunTool(DebuggerRunTool *runTool)
+{
+    DebuggerEngine::setRunTool(runTool);
+    m_qmlEngine->setRunTool(runTool);
+    m_cppEngine->setRunTool(runTool);
 }
 
 void QmlCppEngine::setActiveEngine(DebuggerEngine *engine)
@@ -798,5 +687,11 @@ void QmlCppEngine::setActiveEngine(DebuggerEngine *engine)
     updateViews();
 }
 
+void QmlCppEngine::loadAdditionalQmlStack()
+{
+    if (m_cppEngine)
+        m_cppEngine->loadAdditionalQmlStack();
+}
+
 } // namespace Internal
 } // namespace Debugger
