@@ -30,7 +30,7 @@
 
 #include "../project.h"
 #include "../projectexplorer.h"
-
+#include "../projectexplorerconstants.h"
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/messagemanager.h>
 
@@ -42,6 +42,10 @@
 #include <QMessageBox>
 #include <QVariant>
 
+#ifdef WITH_TESTS
+#include "jsonwizard_test.cpp"
+#endif
+
 namespace ProjectExplorer {
 
 JsonWizard::JsonWizard(QWidget *parent) : Utils::Wizard(parent)
@@ -284,6 +288,10 @@ void JsonWizard::accept()
     emit allDone(m_files);
 
     openFiles(m_files);
+
+    auto node = static_cast<ProjectExplorer::Node*>(value(ProjectExplorer::Constants::PREFERRED_PROJECT_NODE).value<void*>());
+    if (node) // PREFERRED_PROJECT_NODE is not set for newly created projects
+        openProjectForNode(node);
 }
 
 void JsonWizard::reject()
@@ -327,7 +335,7 @@ void JsonWizard::openFiles(const JsonWizard::GeneratorFiles &files)
     bool openedSomething = false;
     foreach (const JsonWizard::GeneratorFile &f, files) {
         const Core::GeneratedFile &file = f.file;
-        if (!QFileInfo(file.path()).exists()) {
+        if (!QFileInfo::exists(file.path())) {
             errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
                                                        "\"%1\" does not exist in the file system.")
                     .arg(QDir::toNativeSeparators(file.path()));
@@ -378,6 +386,24 @@ void JsonWizard::openFiles(const JsonWizard::GeneratorFiles &files)
     }
 }
 
+void JsonWizard::openProjectForNode(Node *node)
+{
+    using namespace Utils;
+
+    ProjectNode *projNode = node->asProjectNode() ? node->asProjectNode() : node->parentProjectNode();
+
+    QTC_ASSERT(projNode, return);
+
+    Utils::optional<FileName> projFilePath = projNode->visibleAfterAddFileAction();
+
+    if (projFilePath && !Core::EditorManager::openEditor(projFilePath.value().toString())) {
+            auto errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
+                                                       "Failed to open an editor for \"%1\".")
+                    .arg(QDir::toNativeSeparators(projFilePath.value().toString()));
+            QMessageBox::warning(nullptr, tr("Cannot Open Project"), errorMessage);
+    }
+}
+
 QString JsonWizard::OptionDefinition::value(Utils::MacroExpander &expander) const
 {
     if (JsonWizard::boolFromVariant(m_evaluate, &expander))
