@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef QBSBUILDSTEP_H
-#define QBSBUILDSTEP_H
+#pragma once
 
 #include "qbsbuildconfiguration.h"
 
@@ -45,9 +44,18 @@ class QbsBuildStep : public ProjectExplorer::BuildStep
 {
     Q_OBJECT
 
+    // used in DebuggerRunConfigurationAspect
+    Q_PROPERTY(bool linkQmlDebuggingLibrary READ isQmlDebuggingEnabled
+               WRITE setQmlDebuggingEnabled NOTIFY qbsConfigurationChanged)
+
 public:
+    enum VariableHandling
+    {
+        PreserveVariables,
+        ExpandVariables
+    };
+
     explicit QbsBuildStep(ProjectExplorer::BuildStepList *bsl);
-    QbsBuildStep(ProjectExplorer::BuildStepList *bsl, const QbsBuildStep *other);
     ~QbsBuildStep() override;
 
     bool init(QList<const BuildStep *> &earlierSteps) override;
@@ -59,26 +67,35 @@ public:
     bool runInGuiThread() const override;
     void cancel() override;
 
-    QVariantMap qbsConfiguration() const;
+    QVariantMap qbsConfiguration(VariableHandling variableHandling) const;
     void setQbsConfiguration(const QVariantMap &config);
 
     bool keepGoing() const;
     bool showCommandLines() const;
     bool install() const;
     bool cleanInstallRoot() const;
+    bool hasCustomInstallRoot() const;
+    Utils::FileName installRoot() const;
     int maxJobs() const;
     QString buildVariant() const;
 
-    bool isQmlDebuggingEnabled() const;
+    void setForceProbes(bool force) { m_forceProbes = force; emit qbsConfigurationChanged(); }
+    bool forceProbes() const { return m_forceProbes; }
 
-    bool fromMap(const QVariantMap &map) override;
-    QVariantMap toMap() const override;
+    void setQmlDebuggingEnabled(bool debug) {
+        m_enableQmlDebugging = debug;
+        emit qbsConfigurationChanged();
+    }
+    bool isQmlDebuggingEnabled() const { return m_enableQmlDebugging; }
 
 signals:
     void qbsConfigurationChanged();
     void qbsBuildOptionsChanged();
 
-private slots:
+private:
+    bool fromMap(const QVariantMap &map) override;
+    QVariantMap toMap() const override;
+
     void buildingDone(bool success);
     void reparsingDone(bool success);
     void handleTaskStarted(const QString &desciption, int max);
@@ -86,7 +103,6 @@ private slots:
     void handleCommandDescriptionReport(const QString &highlight, const QString &message);
     void handleProcessResultReport(const qbs::ProcessResult &result);
 
-private:
     void createTaskAndOutput(ProjectExplorer::Task::TaskType type,
                              const QString &message, const QString &file, int line);
 
@@ -107,6 +123,8 @@ private:
 
     QVariantMap m_qbsConfiguration;
     qbs::BuildOptions m_qbsBuildOptions;
+    bool m_forceProbes = false;
+    bool m_enableQmlDebugging;
 
     // Temporary data:
     QStringList m_changedFiles;
@@ -114,75 +132,22 @@ private:
     QStringList m_products;
 
     QFutureInterface<bool> *m_fi;
-    qbs::BuildJob *m_job;
+    qbs::BuildJob *m_job = nullptr;
     int m_progressBase;
     bool m_lastWasSuccess;
-    ProjectExplorer::IOutputParser *m_parser;
-    bool m_parsingProject;
+    ProjectExplorer::IOutputParser *m_parser = nullptr;
+    bool m_parsingProject = false;
 
     friend class QbsBuildStepConfigWidget;
 };
 
-namespace Ui { class QbsBuildStepConfigWidget; }
-
-class QbsBuildStepConfigWidget : public ProjectExplorer::BuildStepConfigWidget
-{
-    Q_OBJECT
-public:
-    QbsBuildStepConfigWidget(QbsBuildStep *step);
-    ~QbsBuildStepConfigWidget();
-    QString summaryText() const;
-    QString displayName() const;
-
-private slots:
-    void updateState();
-    void updateQmlDebuggingOption();
-    void updatePropertyEdit(const QVariantMap &data);
-
-    void changeBuildVariant(int);
-    void changeShowCommandLines(bool show);
-    void changeKeepGoing(bool kg);
-    void changeJobCount(int count);
-    void changeInstall(bool install);
-    void changeCleanInstallRoot(bool clean);
-    void applyCachedProperties();
-
-    // QML debugging:
-    void linkQmlDebuggingLibraryChecked(bool checked);
-
-private:
-    bool validateProperties(Utils::FancyLineEdit *edit, QString *errorMessage);
-
-    Ui::QbsBuildStepConfigWidget *m_ui;
-
-    QList<QPair<QString, QString> > m_propertyCache;
-    QbsBuildStep *m_step;
-    QString m_summary;
-    bool m_ignoreChange;
-};
-
-class QbsBuildStepFactory : public ProjectExplorer::IBuildStepFactory
+class QbsBuildStepFactory : public ProjectExplorer::BuildStepFactory
 {
     Q_OBJECT
 
 public:
-    explicit QbsBuildStepFactory(QObject *parent = 0);
-
-    // used to show the list of possible additons to a target, returns a list of types
-    QList<Core::Id> availableCreationIds(ProjectExplorer::BuildStepList *parent) const override;
-    // used to translate the types to names to display to the user
-    QString displayNameForId(Core::Id id) const override;
-
-    bool canCreate(ProjectExplorer::BuildStepList *parent, Core::Id id) const override;
-    ProjectExplorer::BuildStep *create(ProjectExplorer::BuildStepList *parent, Core::Id id) override;
-    // used to recreate the runConfigurations when restoring settings
-    bool canRestore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map) const override;
-    ProjectExplorer::BuildStep *restore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map) override;
-    bool canClone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *product) const override;
-    ProjectExplorer::BuildStep *clone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *product) override;
+    QbsBuildStepFactory();
 };
 
 } // namespace Internal
 } // namespace QbsProjectManager
-
-#endif // QBSBUILDSTEP_H
