@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -83,7 +78,7 @@ enum CompletionOrder {
     TypeOrder = -30
 };
 
-static void addCompletion(QList<AssistProposalItem *> *completions,
+static void addCompletion(QList<AssistProposalItemInterface *> *completions,
                           const QString &text,
                           const QIcon &icon,
                           int order,
@@ -100,7 +95,7 @@ static void addCompletion(QList<AssistProposalItem *> *completions,
     completions->append(item);
 }
 
-static void addCompletions(QList<AssistProposalItem *> *completions,
+static void addCompletions(QList<AssistProposalItemInterface *> *completions,
                            const QStringList &newCompletions,
                            const QIcon &icon,
                            int order)
@@ -125,10 +120,10 @@ public:
 class CompletionAdder : public PropertyProcessor
 {
 protected:
-    QList<AssistProposalItem *> *completions;
+    QList<AssistProposalItemInterface *> *completions;
 
 public:
-    CompletionAdder(QList<AssistProposalItem *> *completions,
+    CompletionAdder(QList<AssistProposalItemInterface *> *completions,
                     const QIcon &icon, int order)
         : completions(completions)
         , icon(icon)
@@ -155,7 +150,7 @@ public:
 class LhsCompletionAdder : public CompletionAdder
 {
 public:
-    LhsCompletionAdder(QList<AssistProposalItem *> *completions,
+    LhsCompletionAdder(QList<AssistProposalItemInterface *> *completions,
                        const QIcon &icon,
                        int order,
                        bool afterOn)
@@ -335,6 +330,42 @@ bool isLiteral(AST::Node *ast)
 
 } // Anonymous
 
+QStringList qmlJSAutoComplete(QTextDocument *textDocument,
+                              int position,
+                              const QString &fileName,
+                              TextEditor::AssistReason reason,
+                              const SemanticInfo &info)
+{
+    QStringList list;
+    QmlJSCompletionAssistProcessor processor;
+    QScopedPointer<IAssistProposal> proposal(processor.perform( /* The processor takes ownership. */
+                                                 new QmlJSCompletionAssistInterface(
+                                                     textDocument,
+                                                     position,
+                                                     fileName,
+                                                     reason,
+                                                     info)));
+
+    if (proposal) {
+        GenericProposalModelPtr model = proposal->model().staticCast<GenericProposalModel>();
+
+        int basePosition = proposal->basePosition();
+        const QString prefix = textDocument->toPlainText().mid(basePosition,
+                                                               position - basePosition);
+
+        if (reason == TextEditor::ExplicitlyInvoked) {
+            model->filter(prefix);
+            model->sort(prefix);
+        }
+
+        for (int i = 0; i < model->size(); ++i)
+            list.append(proposal->model()->text(i));
+        list.append(prefix);
+    }
+
+    return list;
+}
+
 } // namesapce QmlJSEditor
 
 Q_DECLARE_METATYPE(QmlJSEditor::CompleteFunctionCall)
@@ -353,12 +384,11 @@ bool QmlJSAssistProposalItem::prematurelyApplies(const QChar &c) const
             || (text().endsWith(QLatin1Char('.')) && c == QLatin1Char('.'));
 }
 
-void QmlJSAssistProposalItem::applyContextualContent(TextEditorWidget *editorWidget,
-                                                      int basePosition) const
+void QmlJSAssistProposalItem::applyContextualContent(TextEditor::TextDocumentManipulatorInterface &manipulator,
+                                                     int basePosition) const
 {
-    const int currentPosition = editorWidget->position();
-    editorWidget->setCursorPosition(basePosition);
-    editorWidget->remove(currentPosition - basePosition);
+    const int currentPosition = manipulator.currentPosition();
+    manipulator.replace(basePosition, currentPosition - basePosition, QString());
 
     QString content = text();
     int cursorOffset = 0;
@@ -377,16 +407,18 @@ void QmlJSAssistProposalItem::applyContextualContent(TextEditorWidget *editorWid
     int replacedLength = 0;
     for (int i = 0; i < replaceable.length(); ++i) {
         const QChar a = replaceable.at(i);
-        const QChar b = editorWidget->characterAt(editorWidget->position() + i);
+        const QChar b = manipulator.characterAt(manipulator.currentPosition() + i);
         if (a == b)
             ++replacedLength;
         else
             break;
     }
-    const int length = editorWidget->position() - basePosition + replacedLength;
-    editorWidget->replace(length, content);
-    if (cursorOffset)
-        editorWidget->setCursorPosition(editorWidget->position() + cursorOffset);
+    const int length = manipulator.currentPosition() - basePosition + replacedLength;
+    manipulator.replace(basePosition, length, content);
+    if (cursorOffset) {
+        manipulator.setCursorPosition(manipulator.currentPosition() + cursorOffset);
+        manipulator.setAutoCompleteSkipPosition(manipulator.currentPosition());
+    }
 }
 
 // -------------------------
@@ -473,11 +505,6 @@ int FunctionHintProposalModel::activeArgument(const QString &prefix) const
 // -----------------------------
 // QmlJSCompletionAssistProvider
 // -----------------------------
-bool QmlJSCompletionAssistProvider::supportsEditor(Core::Id editorId) const
-{
-    return editorId == Constants::C_QMLJSEDITOR_ID;
-}
-
 int QmlJSCompletionAssistProvider::activationCharSequenceLength() const
 {
     return 1;
@@ -511,7 +538,7 @@ QmlJSCompletionAssistProcessor::~QmlJSCompletionAssistProcessor()
 
 IAssistProposal *QmlJSCompletionAssistProcessor::createContentProposal() const
 {
-    GenericProposalModel *model = new QmlJSAssistProposalModel(m_completions);
+    GenericProposalModelPtr model(new QmlJSAssistProposalModel(m_completions));
     return new GenericProposal(m_startPosition, model);
 }
 
@@ -519,10 +546,9 @@ IAssistProposal *QmlJSCompletionAssistProcessor::createHintProposal(
         const QString &functionName, const QStringList &namedArguments,
         int optionalNamedArguments, bool isVariadic) const
 {
-    IFunctionHintProposalModel *model = new FunctionHintProposalModel(
-                functionName, namedArguments, optionalNamedArguments, isVariadic);
-    IAssistProposal *proposal = new FunctionHintProposal(m_startPosition, model);
-    return proposal;
+    FunctionHintProposalModelPtr model(new FunctionHintProposalModel(
+                functionName, namedArguments, optionalNamedArguments, isVariadic));
+    return new FunctionHintProposal(m_startPosition, model);
 }
 
 IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *assistInterface)
@@ -675,7 +701,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
                             nCompletions.append(completion.right(completion.size()-toSkip));
                     completions = nCompletions;
                 }
-                addCompletions(&m_completions, completions, m_interface->fileNameIcon(), KeywordOrder);
+                addCompletions(&m_completions, completions, QmlJSCompletionAssistInterface::fileNameIcon(), KeywordOrder);
                 return createContentProposal();
             }
         }
@@ -703,12 +729,12 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             if (value && completionOperator == QLatin1Char('.')) { // member completion
                 ProcessProperties processProperties(&scopeChain);
                 if (contextFinder.isInLhsOfBinding() && qmlScopeType) {
-                    LhsCompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(),
+                    LhsCompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(),
                                                        PropertyOrder, contextFinder.isAfterOnInLhsOfBinding());
                     processProperties.setEnumerateGeneratedSlots(true);
                     processProperties(value, &completionAdder);
                 } else {
-                    CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
+                    CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
                     processProperties(value, &completionAdder);
                 }
             } else if (value
@@ -757,19 +783,19 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             // id: is special
             AssistProposalItem *idProposalItem = new QmlJSAssistProposalItem;
             idProposalItem->setText(QLatin1String("id: "));
-            idProposalItem->setIcon(m_interface->symbolIcon());
+            idProposalItem->setIcon(QmlJSCompletionAssistInterface::symbolIcon());
             idProposalItem->setOrder(PropertyOrder);
             m_completions.append(idProposalItem);
 
             {
-                LhsCompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(),
+                LhsCompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(),
                                                    PropertyOrder, contextFinder.isAfterOnInLhsOfBinding());
                 processProperties(qmlScopeType, &completionAdder);
             }
 
             if (ScopeBuilder::isPropertyChangesObject(context, qmlScopeType)
                     && scopeChain.qmlScopeObjects().size() == 2) {
-                CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
+                CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
                 processProperties(scopeChain.qmlScopeObjects().first(), &completionAdder);
             }
         }
@@ -789,7 +815,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
                         completion = QString::fromLatin1("\"%1\"").arg(key);
                     else
                         completion = QString::fromLatin1("%1.%2").arg(name, key);
-                    addCompletion(&m_completions, key, m_interface->symbolIcon(),
+                    addCompletion(&m_completions, key, QmlJSCompletionAssistInterface::symbolIcon(),
                                   EnumValueOrder, completion);
                 }
             }
@@ -801,7 +827,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
         if (doQmlTypeCompletion) {
             if (const ObjectValue *qmlTypes = scopeChain.qmlTypes()) {
                 ProcessProperties processProperties(&scopeChain);
-                CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), TypeOrder);
+                CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), TypeOrder);
                 processProperties(qmlTypes, &completionAdder);
             }
         }
@@ -810,32 +836,30 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const AssistInterface *
             // It's a global completion.
             ProcessProperties processProperties(&scopeChain);
             processProperties.setGlobalCompletion(true);
-            CompletionAdder completionAdder(&m_completions, m_interface->symbolIcon(), SymbolOrder);
+            CompletionAdder completionAdder(&m_completions, QmlJSCompletionAssistInterface::symbolIcon(), SymbolOrder);
             processProperties(&completionAdder);
         }
 
         if (doJsKeywordCompletion) {
             // add js keywords
-            addCompletions(&m_completions, Scanner::keywords(), m_interface->keywordIcon(), KeywordOrder);
+            addCompletions(&m_completions, Scanner::keywords(), QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
         }
 
         // add qml extra words
         if (doQmlKeywordCompletion && isQmlFile) {
-            static QStringList qmlWords;
-            static QStringList qmlWordsAlsoInJs;
+            static QStringList qmlWords{
+                QLatin1String("property"),
+                //QLatin1String("readonly")
+                QLatin1String("signal"),
+                QLatin1String("import")
+            };
+            static QStringList qmlWordsAlsoInJs{
+                QLatin1String("default"), QLatin1String("function")
+            };
 
-            if (qmlWords.isEmpty()) {
-                qmlWords << QLatin1String("property")
-                            //<< QLatin1String("readonly")
-                         << QLatin1String("signal")
-                         << QLatin1String("import");
-            }
-            if (qmlWordsAlsoInJs.isEmpty())
-                qmlWordsAlsoInJs << QLatin1String("default") << QLatin1String("function");
-
-            addCompletions(&m_completions, qmlWords, m_interface->keywordIcon(), KeywordOrder);
+            addCompletions(&m_completions, qmlWords, QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
             if (!doJsKeywordCompletion)
-                addCompletions(&m_completions, qmlWordsAlsoInJs, m_interface->keywordIcon(), KeywordOrder);
+                addCompletions(&m_completions, qmlWordsAlsoInJs, QmlJSCompletionAssistInterface::keywordIcon(), KeywordOrder);
         }
 
         m_completions.append(m_snippetCollector.collect());
@@ -932,7 +956,7 @@ bool QmlJSCompletionAssistProcessor::completeFileName(const QString &relativeBas
 
         AssistProposalItem *item = new QmlJSAssistProposalItem;
         item->setText(fileName);
-        item->setIcon(m_interface->fileNameIcon());
+        item->setIcon(QmlJSCompletionAssistInterface::fileNameIcon());
         m_completions.append(item);
     }
 
@@ -970,9 +994,6 @@ QmlJSCompletionAssistInterface::QmlJSCompletionAssistInterface(QTextDocument *te
                                                                const SemanticInfo &info)
     : AssistInterface(textDocument, position, fileName, reason)
     , m_semanticInfo(info)
-    , m_darkBlueIcon(iconForColor(Qt::darkBlue))
-    , m_darkYellowIcon(iconForColor(Qt::darkYellow))
-    , m_darkCyanIcon(iconForColor(Qt::darkCyan))
 {}
 
 const SemanticInfo &QmlJSCompletionAssistInterface::semanticInfo() const
@@ -980,6 +1001,24 @@ const SemanticInfo &QmlJSCompletionAssistInterface::semanticInfo() const
     return m_semanticInfo;
 }
 
+const QIcon &QmlJSCompletionAssistInterface::fileNameIcon()
+{
+    static QIcon darkBlueIcon(iconForColor(Qt::darkBlue));
+    return darkBlueIcon;
+}
+
+const QIcon &QmlJSCompletionAssistInterface::keywordIcon()
+{
+    static QIcon darkYellowIcon(iconForColor(Qt::darkYellow));
+    return darkYellowIcon;
+}
+
+const QIcon &QmlJSCompletionAssistInterface::symbolIcon()
+{
+    static QIcon darkCyanIcon(iconForColor(Qt::darkCyan));
+    return darkCyanIcon;
+}
+
 namespace {
 
 class QmlJSLessThan
@@ -987,7 +1026,7 @@ class QmlJSLessThan
 public:
     QmlJSLessThan(const QString &searchString) : m_searchString(searchString)
     { }
-    bool operator() (const AssistProposalItem *a, const AssistProposalItem *b)
+    bool operator() (const AssistProposalItemInterface *a, const AssistProposalItemInterface *b)
     {
         if (a->order() != b->order())
             return a->order() > b->order();
@@ -995,8 +1034,8 @@ public:
             return true;
         else if (b->text().isEmpty())
             return false;
-        else if (a->data().isValid() != b->data().isValid())
-            return a->data().isValid();
+        else if (a->isValid() != b->isValid())
+            return a->isValid();
         else if (a->text().at(0).isUpper() && b->text().at(0).isLower())
             return false;
         else if (a->text().at(0).isLower() && b->text().at(0).isUpper())
@@ -1021,9 +1060,9 @@ void QmlJSAssistProposalModel::filter(const QString &prefix)
     GenericProposalModel::filter(prefix);
     if (prefix.startsWith(QLatin1String("__")))
         return;
-    QList<AssistProposalItem *> newCurrentItems;
+    QList<AssistProposalItemInterface *> newCurrentItems;
     newCurrentItems.reserve(m_currentItems.size());
-    foreach (AssistProposalItem *item, m_currentItems)
+    foreach (AssistProposalItemInterface *item, m_currentItems)
         if (!item->text().startsWith(QLatin1String("__")))
             newCurrentItems << item;
     m_currentItems = newCurrentItems;
