@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,30 +9,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "refactoringchanges.h"
-#include "basetexteditor.h"
+#include "texteditor.h"
+#include "textdocument.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/dialogs/readonlyfilesdialog.h>
+#include <coreplugin/documentmanager.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <utils/qtcassert.h>
 #include <utils/fileutils.h>
 
@@ -44,7 +42,10 @@
 #include <QDebug>
 #include <QApplication>
 
-using namespace TextEditor;
+using namespace Core;
+using namespace Utils;
+
+namespace TextEditor {
 
 RefactoringChanges::RefactoringChanges()
     : m_data(new RefactoringChangesData)
@@ -57,10 +58,10 @@ RefactoringChanges::RefactoringChanges(RefactoringChangesData *data)
 RefactoringChanges::~RefactoringChanges()
 {}
 
-QList<QPair<QTextCursor, QTextCursor > > RefactoringChanges::rangesToSelections(QTextDocument *document,
-                                                                                const QList<Range> &ranges)
+RefactoringSelections RefactoringChanges::rangesToSelections(QTextDocument *document,
+                                                             const QList<Range> &ranges)
 {
-    QList<QPair<QTextCursor, QTextCursor> > selections;
+    RefactoringSelections selections;
 
     foreach (const Range &range, ranges) {
         QTextCursor start(document);
@@ -94,8 +95,8 @@ bool RefactoringChanges::createFile(const QString &fileName, const QString &cont
     cursor.endEditBlock();
 
     // Write the file to disk:
-    Utils::TextFileFormat format;
-    format.codec = Core::EditorManager::defaultTextCodec();
+    TextFileFormat format;
+    format.codec = EditorManager::defaultTextCodec();
     QString error;
     bool saveOk = format.writeFile(fileName, document->toPlainText(), &error);
     delete document;
@@ -120,25 +121,24 @@ bool RefactoringChanges::removeFile(const QString &fileName) const
     return true;
 }
 
-BaseTextEditorWidget *RefactoringChanges::openEditor(const QString &fileName, bool activate, int line, int column)
+TextEditorWidget *RefactoringChanges::openEditor(const QString &fileName, bool activate, int line, int column)
 {
-    Core::EditorManager::OpenEditorFlags flags = Core::EditorManager::IgnoreNavigationHistory;
+    EditorManager::OpenEditorFlags flags = EditorManager::IgnoreNavigationHistory;
     if (!activate)
-        flags |= Core::EditorManager::DoNotChangeCurrentEditor;
+        flags |= EditorManager::DoNotChangeCurrentEditor;
     if (line != -1) {
         // openEditorAt uses a 1-based line and a 0-based column!
         column -= 1;
     }
-    Core::IEditor *editor = Core::EditorManager::openEditorAt(
-                fileName, line, column, Core::Id(), flags);
+    IEditor *editor = EditorManager::openEditorAt(fileName, line, column, Id(), flags);
 
     if (editor)
-        return qobject_cast<BaseTextEditorWidget *>(editor->widget());
+        return qobject_cast<TextEditorWidget *>(editor->widget());
     else
         return 0;
 }
 
-RefactoringFilePtr RefactoringChanges::file(BaseTextEditorWidget *editor)
+RefactoringFilePtr RefactoringChanges::file(TextEditorWidget *editor)
 {
     return RefactoringFilePtr(new RefactoringFile(editor));
 }
@@ -158,8 +158,8 @@ RefactoringFile::RefactoringFile(QTextDocument *document, const QString &fileNam
     , m_appliedOnce(false)
 { }
 
-RefactoringFile::RefactoringFile(BaseTextEditorWidget *editor)
-    : m_fileName(editor->baseTextDocument()->filePath())
+RefactoringFile::RefactoringFile(TextEditorWidget *editor)
+    : m_fileName(editor->textDocument()->filePath().toString())
     , m_document(0)
     , m_editor(editor)
     , m_openEditor(false)
@@ -178,9 +178,12 @@ RefactoringFile::RefactoringFile(const QString &fileName, const QSharedPointer<R
     , m_editorCursorPosition(-1)
     , m_appliedOnce(false)
 {
-    QList<Core::IEditor *> editors = Core::DocumentModel::editorsForFilePath(fileName);
-    if (!editors.isEmpty())
-        m_editor = qobject_cast<TextEditor::BaseTextEditorWidget *>(editors.first()->widget());
+    QList<IEditor *> editors = DocumentModel::editorsForFilePath(fileName);
+    if (!editors.isEmpty()) {
+        auto editorWidget = qobject_cast<TextEditorWidget *>(editors.first()->widget());
+        if (editorWidget && !editorWidget->isReadOnly())
+            m_editor = editorWidget;
+    }
 }
 
 RefactoringFile::~RefactoringFile()
@@ -208,12 +211,12 @@ QTextDocument *RefactoringFile::mutableDocument() const
         QString fileContents;
         if (!m_fileName.isEmpty()) {
             QString error;
-            QTextCodec *defaultCodec = Core::EditorManager::defaultTextCodec();
-            Utils::TextFileFormat::ReadResult result = Utils::TextFileFormat::readFile(
+            QTextCodec *defaultCodec = EditorManager::defaultTextCodec();
+            TextFileFormat::ReadResult result = TextFileFormat::readFile(
                         m_fileName, defaultCodec,
                         &fileContents, &m_textFileFormat,
                         &error);
-            if (result != Utils::TextFileFormat::ReadSuccess) {
+            if (result != TextFileFormat::ReadSuccess) {
                 qWarning() << "Could not read " << m_fileName << ". Error: " << error;
                 m_textFileFormat.codec = 0;
             }
@@ -241,7 +244,7 @@ QString RefactoringFile::fileName() const
     return m_fileName;
 }
 
-BaseTextEditorWidget *RefactoringFile::editor() const
+TextEditorWidget *RefactoringFile::editor() const
 {
     return m_editor;
 }
@@ -286,7 +289,12 @@ QString RefactoringFile::textOf(const Range &range) const
     return textOf(range.start, range.end);
 }
 
-void RefactoringFile::setChangeSet(const Utils::ChangeSet &changeSet)
+ChangeSet RefactoringFile::changeSet() const
+{
+    return m_changes;
+}
+
+void RefactoringFile::setChangeSet(const ChangeSet &changeSet)
 {
     if (m_fileName.isEmpty())
         return;
@@ -317,17 +325,17 @@ void RefactoringFile::setOpenEditor(bool activate, int pos)
     m_editorCursorPosition = pos;
 }
 
-void RefactoringFile::apply()
+bool RefactoringFile::apply()
 {
     // test file permissions
     if (!QFileInfo(fileName()).isWritable()) {
         const QString &path = fileName();
-        Core::Internal::ReadOnlyFilesDialog roDialog(path, Core::ICore::mainWindow());
+        ReadOnlyFilesDialog roDialog(path, ICore::mainWindow());
         const QString &failDetailText = QApplication::translate("RefactoringFile::apply",
                                                                 "Refactoring cannot be applied.");
         roDialog.setShowFailWarning(true, failDetailText);
-        if (roDialog.exec() == Core::Internal::ReadOnlyFilesDialog::RO_Cancel)
-            return;
+        if (roDialog.exec() == ReadOnlyFilesDialog::RO_Cancel)
+            return false;
     }
 
     // open / activate / goto position
@@ -341,6 +349,8 @@ void RefactoringFile::apply()
         m_editorCursorPosition = -1;
     }
 
+    bool result = true;
+
     // apply changes, if any
     if (m_data && !(m_indentRanges.isEmpty() && m_changes.isEmpty())) {
         QTextDocument *doc = mutableDocument();
@@ -352,10 +362,10 @@ void RefactoringFile::apply()
                 c.beginEditBlock();
 
             // build indent selections now, applying the changeset will change locations
-            const QList<QPair<QTextCursor, QTextCursor> > &indentSelections =
+            const RefactoringSelections &indentSelections =
                     RefactoringChanges::rangesToSelections(doc, m_indentRanges);
             m_indentRanges.clear();
-            const QList<QPair<QTextCursor, QTextCursor> > &reindentSelections =
+            const RefactoringSelections &reindentSelections =
                     RefactoringChanges::rangesToSelections(doc, m_reindentRanges);
             m_reindentRanges.clear();
 
@@ -370,10 +380,14 @@ void RefactoringFile::apply()
 
             // if this document doesn't have an editor, write the result to a file
             if (!m_editor && m_textFileFormat.codec) {
-                QTC_ASSERT(!m_fileName.isEmpty(), return);
+                QTC_ASSERT(!m_fileName.isEmpty(), return false);
                 QString error;
-                if (!m_textFileFormat.writeFile(m_fileName, doc->toPlainText(), &error))
+                // suppress "file has changed" warnings if the file is open in a read-only editor
+                Core::FileChangeBlocker block(m_fileName);
+                if (!m_textFileFormat.writeFile(m_fileName, doc->toPlainText(), &error)) {
                     qWarning() << "Could not apply changes to" << m_fileName << ". Error: " << error;
+                    result = false;
+                }
             }
 
             fileChanged();
@@ -381,12 +395,13 @@ void RefactoringFile::apply()
     }
 
     m_appliedOnce = true;
+    return result;
 }
 
 void RefactoringFile::indentOrReindent(void (RefactoringChangesData::*mf)(const QTextCursor &,
                                                                           const QString &,
-                                                                          const BaseTextDocument *) const,
-                                       const QList<QPair<QTextCursor, QTextCursor> > &ranges)
+                                                                          const TextDocument *) const,
+                                       const RefactoringSelections &ranges)
 {
     typedef QPair<QTextCursor, QTextCursor> CursorPair;
 
@@ -394,7 +409,7 @@ void RefactoringFile::indentOrReindent(void (RefactoringChangesData::*mf)(const
         QTextCursor selection(p.first.document());
         selection.setPosition(p.first.position());
         selection.setPosition(p.second.position(), QTextCursor::KeepAnchor);
-        ((*m_data).*(mf))(selection, m_fileName, m_editor ? m_editor->baseTextDocument() : 0);
+        ((*m_data).*(mf))(selection, m_fileName, m_editor ? m_editor->textDocument() : 0);
     }
 }
 
@@ -407,12 +422,12 @@ void RefactoringFile::fileChanged()
 RefactoringChangesData::~RefactoringChangesData()
 {}
 
-void RefactoringChangesData::indentSelection(const QTextCursor &, const QString &, const BaseTextDocument *) const
+void RefactoringChangesData::indentSelection(const QTextCursor &, const QString &, const TextDocument *) const
 {
     qWarning() << Q_FUNC_INFO << "not implemented";
 }
 
-void RefactoringChangesData::reindentSelection(const QTextCursor &, const QString &, const BaseTextDocument *) const
+void RefactoringChangesData::reindentSelection(const QTextCursor &, const QString &, const TextDocument *) const
 {
     qWarning() << Q_FUNC_INFO << "not implemented";
 }
@@ -420,3 +435,5 @@ void RefactoringChangesData::reindentSelection(const QTextCursor &, const QStrin
 void RefactoringChangesData::fileChanged(const QString &)
 {
 }
+
+} // namespace TextEditor
