@@ -328,11 +328,10 @@ void SnippetsSettingsPagePrivate::configureUi(QWidget *w)
 {
     m_ui.setupUi(w);
 
-    const QList<SnippetProvider *> &providers = SnippetProvider::snippetProviders();
-    for (SnippetProvider *provider : providers) {
-        m_ui.groupCombo->addItem(provider->displayName(), provider->groupId());
+    for (const SnippetProvider &provider : SnippetProvider::snippetProviders()) {
+        m_ui.groupCombo->addItem(provider.displayName(), provider.groupId());
         SnippetEditorWidget *snippetEditor = new SnippetEditorWidget(w);
-        provider->decorateEditor(snippetEditor);
+        SnippetProvider::decorateEditor(snippetEditor, provider.groupId());
         m_ui.snippetsEditorStack->insertWidget(m_ui.groupCombo->count() - 1, snippetEditor);
         connect(snippetEditor, &SnippetEditorWidget::snippetContentChanged,
                 this, &SnippetsSettingsPagePrivate::setSnippetContent);
@@ -536,16 +535,12 @@ void SnippetsSettingsPagePrivate::setSnippetContent()
 
 void SnippetsSettingsPagePrivate::decorateEditors(const TextEditor::FontSettings &fontSettings)
 {
-    const QList<SnippetProvider *> &providers = SnippetProvider::snippetProviders();
     for (int i = 0; i < m_ui.groupCombo->count(); ++i) {
         SnippetEditorWidget *snippetEditor = editorAt(i);
         snippetEditor->textDocument()->setFontSettings(fontSettings);
         const QString &id = m_ui.groupCombo->itemData(i).toString();
         // This list should be quite short... Re-iterating over it is ok.
-        foreach (const SnippetProvider *provider, providers) {
-            if (provider->groupId() == id)
-                provider->decorateEditor(snippetEditor);
-        }
+        SnippetProvider::decorateEditor(snippetEditor, id);
     }
 }
 
