@@ -33,6 +33,7 @@
 #include <projectpartpch.h>
 
 #include <QCryptographicHash>
+#include <QDateTime>
 #include <QFile>
 #include <QTemporaryFile>
 
@@ -80,11 +81,18 @@ template <typename Source,
           typename Target>
 void append(Target &target, Source &source)
 {
-    using ValueType = typename Target::value_type;
     target.reserve(target.size() + source.size());
 
     for (auto &&entry : source)
-        target.push_back(ValueType(entry));
+        target.emplace_back(entry);
+}
+
+void appendFilePathId(Utils::PathStringVector &target,
+                      const ClangBackEnd::FilePathIds &source,
+                      const ClangBackEnd::FilePathCachingInterface &filePathCache)
+{
+    for (FilePathId id : source)
+        target.emplace_back(filePathCache.filePath(id).path());
 }
 
 template <typename GetterFunction>
@@ -110,7 +118,7 @@ void generateGlobal(Container &entries,
     entries.reserve(entries.capacity() + globalCount(projectsParts, getterFunction));
 
     for (const V2::ProjectPartContainer &projectPart : projectsParts) {
-        const auto &projectPartPaths = getterFunction(projectPart);
+        auto &&projectPartPaths = getterFunction(projectPart);
 
         append(entries, projectPartPaths);
     };
@@ -138,7 +146,7 @@ Utils::PathStringVector generatedFilePaths(const V2::FileContainers &generaredFi
     generaredFilePaths.reserve(generaredFiles.size());
 
     for (const V2::FileContainer &generatedFile : generaredFiles)
-        generaredFilePaths.push_back(generatedFile.filePath().path());
+        generaredFilePaths.push_back(generatedFile.filePath.path());
 
     return generaredFilePaths;
 }
@@ -147,9 +155,8 @@ Utils::PathStringVector generatedFilePaths(const V2::FileContainers &generaredFi
 
 Utils::PathStringVector PchCreator::generateGlobalHeaderPaths() const
 {
-    auto includeFunction = [] (const V2::ProjectPartContainer &projectPart)
-            -> const Utils::PathStringVector & {
-        return projectPart.headerPaths();
+    auto includeFunction = [&] (const V2::ProjectPartContainer &projectPart) {
+        return m_filePathCache.filePaths(projectPart.headerPathIds);
     };
 
     Utils::PathStringVector headerPaths = generateGlobal<Utils::PathStringVector>(m_projectParts,
@@ -167,9 +174,8 @@ Utils::PathStringVector PchCreator::generateGlobalHeaderPaths() const
 
 Utils::PathStringVector PchCreator::generateGlobalSourcePaths() const
 {
-    auto sourceFunction = [] (const V2::ProjectPartContainer &projectPart)
-            -> const Utils::PathStringVector & {
-        return projectPart.sourcePaths();
+    auto sourceFunction = [&] (const V2::ProjectPartContainer &projectPart) {
+        return m_filePathCache.filePaths(projectPart.sourcePathIds);
     };
 
     return generateGlobal<Utils::PathStringVector>(m_projectParts, sourceFunction);
@@ -191,7 +197,7 @@ Utils::SmallStringVector PchCreator::generateGlobalArguments() const
 
     auto argumentFunction = [] (const V2::ProjectPartContainer &projectPart)
             -> const Utils::SmallStringVector & {
-        return projectPart.arguments();
+        return projectPart.arguments;
     };
 
     generateGlobal(arguments, m_projectParts, argumentFunction);
@@ -206,7 +212,7 @@ Utils::SmallStringVector PchCreator::generateGlobalCommandLine() const
 
     auto argumentFunction = [] (const V2::ProjectPartContainer &projectPart)
             -> const Utils::SmallStringVector & {
-        return projectPart.arguments();
+        return projectPart.arguments;
     };
 
     generateGlobal(commandLine, m_projectParts, argumentFunction);
@@ -259,7 +265,7 @@ namespace {
 
 std::size_t contentSize(const FilePaths &includes)
 {
-    auto countIncludeSize = [] (std::size_t size, const Utils::PathString &include) {
+    auto countIncludeSize = [] (std::size_t size, const auto &include) {
         return size + include.size();
     };
 
@@ -275,7 +281,7 @@ Utils::SmallString PchCreator::generatePchIncludeFileContent(const FilePathIds &
 
     fileContent.reserve(includes.size() * lineTemplateSize + contentSize(includes));
 
-    for (const Utils::PathString &include : includes)
+    for (const Utils::SmallStringView &include : includes)
         fileContent += {"#include \"", include, "\"\n"};
 
     return fileContent;
@@ -318,10 +324,10 @@ namespace {
 
 void hashProjectPart(QCryptographicHash &hash, const V2::ProjectPartContainer &projectPart)
 {
-    const auto &projectPartId = projectPart.projectPartId();
+    const auto &projectPartId = projectPart.projectPartId;
     hash.addData(projectPartId.data(), projectPartId.size());
 
-    for (const auto &argument : projectPart.arguments())
+    for (const auto &argument : projectPart.arguments)
         hash.addData(argument.data(), argument.size());
 }
 }
@@ -368,7 +374,7 @@ Utils::SmallString PchCreator::generateGlobalPchFilePath() const
 Utils::SmallStringVector PchCreator::generateProjectPartCommandLine(
         const V2::ProjectPartContainer &projectPart) const
 {
-    const Utils::SmallStringVector &arguments = projectPart.arguments();
+    const Utils::SmallStringVector &arguments = projectPart.arguments;
 
     Utils::SmallStringVector commandLine;
     commandLine.reserve(arguments.size() + 1);
@@ -394,11 +400,14 @@ Utils::PathStringVector PchCreator::generateProjectPartHeaders(
         const V2::ProjectPartContainer &projectPart) const
 {
    Utils::PathStringVector headerPaths;
-   headerPaths.reserve(projectPart.headerPaths().size() + m_generatedFiles.size());
+   headerPaths.reserve(projectPart.headerPathIds.size() + m_generatedFiles.size());
 
-   std::copy(projectPart.headerPaths().begin(),
-             projectPart.headerPaths().end(),
-             std::back_inserter(headerPaths));
+   std::transform(projectPart.headerPathIds.begin(),
+                  projectPart.headerPathIds.end(),
+                  std::back_inserter(headerPaths),
+                  [&] (FilePathId filePathId) {
+       return m_filePathCache.filePath(filePathId);
+   });
 
    Utils::PathStringVector generatedPath = generatedFilePaths(m_generatedFiles);
 
@@ -439,7 +448,7 @@ Utils::SmallString concatContent(const Utils::PathStringVector &paths, std::size
 }
 
 Utils::SmallString PchCreator::generateProjectPartHeaderAndSourcesContent(
-        const V2::ProjectPartContainer &projectPart)
+        const V2::ProjectPartContainer &projectPart) const
 {
     Utils::PathStringVector paths = generateProjectPartHeaderAndSourcePaths(projectPart);
 
@@ -447,18 +456,18 @@ Utils::SmallString PchCreator::generateProjectPartHeaderAndSourcesContent(
 }
 
 Utils::PathStringVector PchCreator::generateProjectPartHeaderAndSourcePaths(
-        const V2::ProjectPartContainer &projectPart)
+        const V2::ProjectPartContainer &projectPart) const
 {
     Utils::PathStringVector includeAndSources;
-    includeAndSources.reserve(projectPart.headerPaths().size() + projectPart.sourcePaths().size());
+    includeAndSources.reserve(projectPart.headerPathIds.size() + projectPart.sourcePathIds.size());
 
-    append(includeAndSources, projectPart.headerPaths());
-    append(includeAndSources, projectPart.sourcePaths());
+    appendFilePathId(includeAndSources, projectPart.headerPathIds, m_filePathCache);
+    appendFilePathId(includeAndSources, projectPart.sourcePathIds, m_filePathCache);
 
     return includeAndSources;
 }
 
-FilePathIds PchCreator::generateProjectPartPchIncludes(
+std::pair<FilePathIds,FilePathIds> PchCreator::generateProjectPartPchIncludes(
         const V2::ProjectPartContainer &projectPart) const
 {
     Utils::SmallString jointedFileContent = generateProjectPartHeaderAndSourcesContent(projectPart);
@@ -483,7 +492,7 @@ FilePathIds PchCreator::generateProjectPartPchIncludes(
 
     jointFile->remove();
 
-    return  collector.takeIncludeIds();
+    return  {collector.takeIncludeIds(), collector.takeTopIncludeIds()};
 }
 
 Utils::SmallString PchCreator::generateProjectPathPchHeaderFilePath(
@@ -527,7 +536,7 @@ Utils::SmallStringVector PchCreator::generateProjectPartPchCompilerArguments(
 Utils::SmallStringVector PchCreator::generateProjectPartClangCompilerArguments(
         const V2::ProjectPartContainer &projectPart) const
 {
-    Utils::SmallStringVector compilerArguments = projectPart.arguments().clone();
+    Utils::SmallStringVector compilerArguments = projectPart.arguments.clone();
     const auto pchArguments = generateProjectPartPchCompilerArguments(projectPart);
 
     append(compilerArguments, pchArguments);
@@ -537,16 +546,19 @@ Utils::SmallStringVector PchCreator::generateProjectPartClangCompilerArguments(
 
 IdPaths PchCreator::generateProjectPartPch(const V2::ProjectPartContainer &projectPart)
 {
-    auto includes = generateProjectPartPchIncludes(projectPart);
-    auto content = generatePchIncludeFileContent(includes);
+    long long lastModified = QDateTime::currentSecsSinceEpoch();
+    FilePathIds allExternalIncludes;
+    FilePathIds topExternalIncludes;
+    std::tie(allExternalIncludes, topExternalIncludes) = generateProjectPartPchIncludes(projectPart);
+    auto content = generatePchIncludeFileContent(topExternalIncludes);
     auto pchIncludeFilePath = generateProjectPathPchHeaderFilePath(projectPart);
     auto pchFilePath = generateProjectPartPchFilePath(projectPart);
     generateFileWithContent(pchIncludeFilePath, content);
 
     generatePch(generateProjectPartClangCompilerArguments(projectPart),
-                {projectPart.projectPartId().clone(), std::move(pchFilePath)});
+                {projectPart.projectPartId.clone(), std::move(pchFilePath), lastModified});
 
-    return {projectPart.projectPartId().clone(), std::move(includes)};
+    return {projectPart.projectPartId.clone(), std::move(allExternalIncludes)};
 }
 
 void PchCreator::generatePchs()
