@@ -27,16 +27,13 @@ source("../../shared/qtcreator.py")
 
 def main():
     global tmpSettingsDir, availableBuildSystems
-    qtVersionsForQuick = ["5.3"]
     availableBuildSystems = ["qmake", "Qbs"]
-    if platform.system() != 'Darwin':
-        qtVersionsForQuick.append("5.4")
     if which("cmake"):
         availableBuildSystems.append("CMake")
     else:
         test.warning("Could not find cmake in PATH - several tests won't run without.")
 
-    startApplication("qtcreator" + SettingsPath)
+    startQC()
     if not startedWithoutPluginError():
         return
     kits = getConfiguredKits()
@@ -48,7 +45,6 @@ def main():
     projects = catModel.index(0, 0)
     test.compare("Projects", str(projects.data()))
     comboBox = findObject(":New.comboBox_QComboBox")
-    targets = zip(*kits.values())[0]
     test.verify(comboBox.enabled, "Verifying whether combobox is enabled.")
     test.compare(comboBox.currentText, "All Templates")
     try:
@@ -73,7 +69,8 @@ def main():
         category = current.keys()[0]
         template = current.values()[0]
         displayedPlatforms = __createProject__(category, template)
-        if template == "Qt Quick Application" or template == "Qt Quick Controls Application":
+        if template.startswith("Qt Quick Application - "):
+            qtVersionsForQuick = ["5.6", "5.10"] if template == "Qt Quick Application - Empty" else ["5.10"]
             for counter, qtVersion in enumerate(qtVersionsForQuick):
                 def additionalFunc(displayedPlatforms, qtVersion):
                     requiredQtVersion = __createProjectHandleQtQuickSelection__(qtVersion)
@@ -84,43 +81,31 @@ def main():
                 if counter < len(qtVersionsForQuick) - 1:
                     displayedPlatforms = __createProject__(category, template)
             continue
-        elif template == "Qt Quick Controls 2 Application": # needs a Qt5.7
-            def additionalFunc(displayedPlatforms):
-                clickButton(waitForObject(":Next_QPushButton")) # ignore this details page for now
-            handleBuildSystemVerifyKits(category, template, kits, displayedPlatforms, additionalFunc)
-            continue
-        elif template.startswith("Plain C"):
-            handleBuildSystemVerifyKits(category, template, kits, displayedPlatforms)
-            continue
-
         handleBuildSystemVerifyKits(category, template, kits, displayedPlatforms)
 
     invokeMenuItem("File", "Exit")
 
 def verifyKitCheckboxes(kits, displayedPlatforms):
     waitForObject("{type='QLabel' unnamed='1' visible='1' text='Kit Selection'}")
-    availableCheckboxes = filter(visibleCheckBoxExists, kits.keys())
+    availableCheckboxes = frozenset(filter(visibleCheckBoxExists, kits.keys()))
     # verification whether expected, found and configured match
-    for t in kits:
-        if t in displayedPlatforms:
-            if t in availableCheckboxes:
-                test.passes("Found expected kit '%s' on 'Kit Selection' page." % t)
-                availableCheckboxes.remove(t)
-            else:
-                test.fail("Expected kit '%s' missing on 'Kit Selection' page." % t)
-        else:
-            if t in availableCheckboxes:
-                test.fail("Kit '%s' found on 'Kit Selection' page - but was not expected!" % t)
-            else:
-                test.passes("Irrelevant kit '%s' not found on 'Kit Selection' page." % t)
-    if len(availableCheckboxes) != 0:
-        test.fail("Found unexpected additional kit(s) %s on 'Kit Selection' page."
-                  % str(availableCheckboxes))
+
+    expectedShownKits = availableCheckboxes.intersection(displayedPlatforms)
+    unexpectedShownKits = availableCheckboxes.difference(displayedPlatforms)
+    missingKits = displayedPlatforms.difference(availableCheckboxes)
+
+    test.log("Expected kits shown on 'Kit Selection' page:\n%s" % "\n".join(expectedShownKits))
+    if len(unexpectedShownKits):
+        test.fail("Kits found on 'Kit Selection' page but not expected:\n%s"
+                  % "\n".join(unexpectedShownKits))
+    if len(missingKits):
+        test.fail("Expected kits missing on 'Kit Selection' page:\n%s"
+                  % "\n".join(missingKits))
 
 def handleBuildSystemVerifyKits(category, template, kits, displayedPlatforms,
                                 specialHandlingFunc = None, *args):
     global availableBuildSystems
-    combo = "{name='BuildSystem' type='Utils::TextFieldComboBox' visible='1'}"
+    combo = "{name='BuildSystem' type='QComboBox' visible='1'}"
     try:
         waitForObject(combo, 2000)
         skipBuildsystemChooser = False
@@ -139,7 +124,9 @@ def handleBuildSystemVerifyKits(category, template, kits, displayedPlatforms,
         test.log("Using build system '%s'" % buildSystem)
         selectFromCombo(combo, buildSystem)
         clickButton(waitForObject(":Next_QPushButton"))
-        if specialHandlingFunc:
+        if template == "Qt Quick Application - Scroll":
+            clickButton(waitForObject(":Next_QPushButton"))
+        elif specialHandlingFunc:
             specialHandlingFunc(displayedPlatforms, *args)
         verifyKitCheckboxes(kits, displayedPlatforms)
         safeClickButton("Cancel")
